require "spec_helper"

describe Nfe::Helpers::NfeKey do
  let(:nfe_key) { "43091285961757000102551140912140011007055476" }
  subject { described_class.parse nfe_key }

  describe "#uf" do
    it "returns the UF" do
      expect(subject.uf).to eql "RS"
    end
  end

  describe "#year" do
    it "returns the year" do
      expect(subject.year).to eql "09"
    end
  end

  describe "#month" do
    it "returns the month" do
      expect(subject.month).to eql "12"
    end
  end

  describe "#cnpj" do
    it "returns the CNPJ" do
      expect(subject.cnpj).to eql "85961757000102"
    end
  end

  describe "#model" do
    it "returns the model" do
      expect(subject.model).to eql 55
    end
  end

  describe "#series" do
    it "returns the series" do
      expect(subject.series).to eql 114
    end
  end

  describe "#number" do
    it "returns the number" do
      expect(subject.number).to eql 91214001
    end
  end

  describe "#emission_kind" do
    it "returns the emission kind" do
      expect(subject.emission_kind).to eql 1
    end
  end

  describe "#code" do
    it "returns the code" do
      expect(subject.code).to eql 705547
    end
  end

  describe "#check_digit" do
    it "returns the check digit" do
      expect(subject.check_digit).to eql 6
    end
  end

  describe "#key" do
    subject do
      described_class.new("RS", Date.new(2009,12,1), "85961757000102",
        114, 91214001, 705547)
    end

    it "returns the key" do
      expect(subject.key).to eql nfe_key
    end
  end

  describe "parse" do
    context "when nfe key is invalid" do
      let(:nfe_key) { nil }

      it "raise a exception" do
        expect { described_class.parse nfe_key }.to raise_error(NfeKeyError)
      end
    end
  end
end
