require "spec_helper"

describe Nfe::Helpers::XmlModel::AttributeBase do
  let(:options) { {} }

  subject { described_class.new(:zaz, options) }

  describe "#name" do
    it "returns the attribute's name" do
      expect(subject.name).to eq :zaz
    end
  end

  describe "#type" do
    let(:options) { { type: :float } }

    it "returns the attribute's type" do
      expect(subject.type).to be :float
    end

    context "when attribute's type is ommited" do
      let(:options) { {} }

      it "returns text" do
        expect(subject.type).to be :text
      end
    end
  end

  describe "#format" do
    context "when format is present" do
      let(:options) { { format: "%0.3f" } }

      it "returns the attribute's format" do
        expect(subject.format).to eq "%0.3f"
      end
    end
  end

  describe "#xml_alias" do
    context "when xml alias is present" do
      let(:options) { { xml_alias: "zazAlias" } }

      it "returns the xml's alias" do
        expect(subject.xml_alias).to eql "zazAlias"
      end
    end

    context "when xml alias ins't present" do
      it "returns just the attribute" do
        expect(subject.xml_alias).to eql "zaz"
      end
    end
  end

  describe "#klass" do
    let(:options) { { klass: "FOO" } }

    it "returns the attribute's klass" do
      expect(subject.klass).to eq "FOO"
    end
  end
end
