require "spec_helper"

class ZazCollection
  include Nfe::Helpers::XmlModel::NfeModel
  attr_xml :value, type: :text
end

class DooCollection
  attr_accessor :zaz
end

describe Nfe::Helpers::XmlModel::AttributeCollection do
  let!(:doo) { DooCollection.new }

  subject { described_class.new(:zaz, { klass: "ZazCollection" }) }

  describe "#parse" do
    let(:doc) { Nokogiri::XML("<doo><zaz><value>1</value></zaz><zaz><value>2</value></zaz></doo>") }

    before { subject.parse doo, doc }

    it "read like a collection attribute" do
      expect(doo.zaz[0].value).to eql "1"
      expect(doo.zaz[1].value).to eql "2"
    end
  end

  describe "#add_to_xml" do
    let(:builder) { Builder::XmlMarkup.new }

    before do
      doo.zaz = []
      ["1", "2"].each do |value|
        zaz = ZazCollection.new
        zaz.value = value
        doo.zaz.push zaz
      end
    end

    it "write like a node attribute" do
      subject.add_to_xml doo, builder, :zaz

      expect(builder.target!).to eq "<zaz><value>1</value></zaz><zaz><value>2</value></zaz>"
    end
  end
end
