require "spec_helper"

class DooDate
  attr_accessor :zaz
end

describe Nfe::Helpers::XmlModel::AttributeDate do
  let(:doo) { DooDate.new }

  subject { described_class.new(:zaz) }

  describe "#format" do
    context "when attribute's format is ommited" do
      it "returns the default" do
        expect(subject.format).to eq "%Y-%m-%d"
      end
    end
  end

  describe "#parse" do
    let(:doc) { Nokogiri::XML("<zaz>2004-10-24</zaz>") }

    before { subject.parse doo, doc }

    it "read like a date attribute" do
      expect(doo.zaz).to eql Date.parse("2004-10-24")
    end
  end

  describe "#add_to_xml" do
    let(:xml) { Builder::XmlMarkup.new }
    before { doo.zaz = Date.parse("2005-08-23") }

    it "write like a date attribute" do
      subject.add_to_xml doo, xml, :zaz

      expect(xml.target!).to eq "<zaz>2005-08-23</zaz>"
    end
  end
end
