require "spec_helper"

class DooDatetime
  attr_accessor :zaz
end

describe Nfe::Helpers::XmlModel::AttributeDatetime do
  let(:doo) { DooDatetime.new }

  subject { described_class.new(:zaz) }

  describe "#format" do
    context "when attribute's format is ommited" do
      it "returns the default" do
        expect(subject.format).to eq "%Y-%m-%dT%H:%M:%S%:z"
      end
    end
  end

  describe "#parse" do
    let(:doc) { Nokogiri::XML("<zaz>2004-10-24T02:01:12-03:00</zaz>") }

    before { subject.parse doo, doc }

    it "read like a datetime attribute" do
      expect(doo.zaz).to eql DateTime.strptime("2004-10-24T02:01:12-03:00",
        "%Y-%m-%dT%H:%M:%S%:z")
    end
  end

  describe "#add_to_xml" do
    let(:xml) { Builder::XmlMarkup.new }
    before { doo.zaz = DateTime.strptime("2004-10-24T02:01:12-03:00", "%Y-%m-%dT%H:%M:%S%:z") }

    it "write like a datetime attribute" do
      subject.add_to_xml doo, xml, :zaz

      expect(xml.target!).to eq "<zaz>2004-10-24T02:01:12-03:00</zaz>"
    end
  end
end
