require "spec_helper"

class DooFloat
  attr_accessor :zaz
end

describe Nfe::Helpers::XmlModel::AttributeFloat do
  let(:doo) { DooFloat.new }

  subject { described_class.new(:zaz) }

  describe "#format" do
    context "when attribute's format is ommited" do
      it "returns the default" do
        expect(subject.format).to eq "%0.2f"
      end
    end
  end

  describe "#parse" do
    let(:doc) { Nokogiri::XML("<zaz>10.47</zaz>") }

    before { subject.parse doo, doc }

    it "read like a float attribute" do
      expect(doo.zaz).to eql 10.47
    end
  end

  describe "#add_to_xml" do
    let(:xml) { Builder::XmlMarkup.new }
    before { doo.zaz = 10.4 }

    it "write like a float attribute" do
      subject.add_to_xml doo, xml, :zaz

      expect(xml.target!).to eq "<zaz>10.40</zaz>"
    end
  end
end
