require "spec_helper"

class ZazNode
  include Nfe::Helpers::XmlModel::NfeModel
  attr_xml :value, type: :text
end

class DooNode
  attr_accessor :zaz
end

describe Nfe::Helpers::XmlModel::AttributeNode do
  let(:doo) { DooNode.new }

  subject { described_class.new(:zaz) }

  describe "#parse" do
    let(:doc) { Nokogiri::XML("<doo><zaz><value>val</value></zaz></doo>") }

    before do
      doo.zaz = ZazNode.new
      subject.parse doo, doc
    end

    it "read like a node attribute" do
      expect(doo.zaz.value).to eq "val"
    end
  end

  describe "#add_to_xml" do
    let(:xml) { Builder::XmlMarkup.new }

    before do
      doo.zaz = ZazNode.new
      doo.zaz.value = "zaz"
    end

    it "write like a node attribute" do
      subject.add_to_xml doo, xml, :zaz

      expect(xml.target!).to eq "<zaz><value>zaz</value></zaz>"
    end
  end
end
