require "spec_helper"

describe Nfe::Models::NFe do
  subject { build :NFe }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  context "when infNFe is invalid" do
    before { subject.infNFe.ide.uf = "" }

    describe "#valid?" do
      it { is_expected.to_not be_valid }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<NFe xmlns=\"http://www.portalfiscal.inf.br/nfe\">"
      xml +=   "<infNFe versao=\"3.10\" Id=\"NFe42130901144012000152551231234567891123456785\">"
      xml +=     "#{build(:infNFe).to_xml}"
      xml +=   "</infNFe>"
      xml +=   "<Signature xmlns=\"#{Nfe::Models::Signature.namespace}\">"
      xml +=     "#{build(:signature).value}"
      xml +=   "</Signature>"
      xml += "</NFe>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::NFe.new }
      before { subject.parse!("<NFe xmlns='#{Nfe::Models::NFe.namespace}'>#{xml}</NFe>") }

      it "parses xml" do
        expect(subject.infNFe.to_xml).to eql build(:infNFe).to_xml
        expect(subject.signature.value).to eql build(:signature).value
      end

      it { is_expected.to be_valid }
    end
  end
end
