require "spec_helper"
require_relative "shared_examples_for_tdec_1302"

describe Nfe::Models::Avulsa do
  subject { build(:avulsa) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:xOrgao).is_at_least(1).is_at_most(60) }

  it { is_expected.to ensure_length_of(:matr).is_at_most(60) }

  it { is_expected.to ensure_length_of(:xAgente).is_at_least(1).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:fone) }
  it { is_expected.to ensure_length_of(:fone).is_at_least(6).is_at_most(14) }
  it { is_expected.to validate_numericality_of(:fone).only_integer }

  it { is_expected.to ensure_length_of(:uf).is_equal_to(2) }

  it { is_expected.to allow_value("", nil).for(:nDAR) }
  it { is_expected.to ensure_length_of(:nDAR).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:dEmi) }

  it { is_expected.to allow_value("", nil).for(:vDAR) }
  it { is_expected.to validate_numericality_of(:vDAR ) }

  it { is_expected.to ensure_length_of(:repEmi).is_at_least(1).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:dPag) }

  it_should_behave_like "a numeric TDec_1302Opc field", :vDAR

  describe "#CNPJ" do
    context "when number is invalid" do
      before { subject.cnpj = "04001155000103" }
      it { is_expected.not_to be_valid }
    end

    it {
      is_expected.to ensure_length_of(:cnpj)
        .is_equal_to(14)
        .with_message(/não possui o tamanho esperado/)
    }
  end

  describe "#present?" do
    subject { Nfe::Models::Avulsa.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.xAgente = "James Bond" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<CNPJ>01144012000152</CNPJ>"
      xml += "<xOrgao>Secretaria de Segurança Pública</xOrgao>"
      xml += "<matr>PN13654987</matr>"
      xml += "<xAgente>James Bond</xAgente>"
      xml += "<fone>4832630000</fone>"
      xml += "<UF>SC</UF>"
      xml += "<nDAR>11213551684313513516846832136518</nDAR>"
      xml += "<dEmi>2013-10-20</dEmi>"
      xml += "<vDAR>123456.78</vDAR>"
      xml += "<repEmi>Tributos</repEmi>"
      xml += "<dPag>2013-11-25</dPag>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Avulsa.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.cnpj ).to eql "01144012000152"
        expect(subject.xOrgao).to eql "Secretaria de Segurança Pública"
        expect(subject.matr).to eql "PN13654987"
        expect(subject.xAgente).to eql "James Bond"
        expect(subject.fone).to eql "4832630000"
        expect(subject.uf).to eql "SC"
        expect(subject.nDAR).to eql "11213551684313513516846832136518"
        expect(subject.dEmi).to eql Date.parse("2013-10-20")
        expect(subject.vDAR).to eql 123456.78
        expect(subject.repEmi).to eql "Tributos"
        expect(subject.dPag).to eql Date.parse("2013-11-25")
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<CNPJ>01144012000152</CNPJ>"
      xml += "<xOrgao>Secretaria de Segurança Pública</xOrgao>"
      xml += "<matr>PN13654987</matr>"
      xml += "<xAgente>James Bond</xAgente>"
      xml += "<UF>SC</UF>"
      xml += "<repEmi>Tributos</repEmi>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:avulsa, fone: nil, nDAR: nil, dEmi: nil, vDAR: nil, dPag: nil)
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Avulsa.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cnpj ).to eql "01144012000152"
        expect(subject.xOrgao).to eql "Secretaria de Segurança Pública"
        expect(subject.matr).to eql "PN13654987"
        expect(subject.xAgente).to eql "James Bond"
        expect(subject.uf).to eql "SC"
        expect(subject.repEmi).to eql "Tributos"
      end

      it { is_expected.to be_valid }
    end
  end
end
