require "spec_helper"

describe Nfe::Models::Bill::Cobr do
  subject { build(:cobr) }

  context "when fat is present and invalid" do
    it "returns an error on fat" do
      subject.fat.vOrig = 999_999_999_999_999.99
      subject.valid?

      expect(subject.errors.to_json).to match(/fat/)
    end
  end

  context "when dup is present and invalid" do
    it "returns an error on dup" do
      subject.dup[0].vDup = 999_999_999_999_999.99
      subject.valid?

      expect(subject.errors.to_json).to match(/dup/)
    end
  end

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  describe "#dup" do
    it "should be less than or equal to 120" do
      121.times { subject.dup << build(:dup) }
      subject.valid?
      expect(subject.errors[:dup].to_s).to match(/deve ser menor ou igual a 120/)
    end
  end

  describe "#present?" do
    subject { Nfe::Models::Bill::Cobr.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.dup << "X"  }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml = "<fat>#{build(:fat).to_xml}</fat>"
      3.times { |n| xml += "<dup>#{build(:dup, nDup: n).to_xml}</dup>" }
      xml
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Bill::Cobr.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.fat.to_xml).to eql build(:fat).to_xml
        expect(subject.dup[0].to_xml).to eql build(:dup, nDup: 0).to_xml
        expect(subject.dup[1].to_xml).to eql build(:dup, nDup: 1).to_xml
        expect(subject.dup[2].to_xml).to eql build(:dup, nDup: 2).to_xml
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    subject { Nfe::Models::Bill::Cobr.new }

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to_not be_present
      end
    end

    describe "#parse!" do
      before { subject.parse!("<root></root>") }

      it "parses xml" do
        expect(subject.fat).to_not be_present
        expect(subject.dup).to be_blank
      end

      it { is_expected.to be_valid }
    end
  end
end
