require "spec_helper"

describe Nfe::Models::Bill::Dup do
  subject { build(:dup) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to allow_value("", nil).for(:nDup) }
  it { is_expected.to ensure_length_of(:nDup).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:dVenc) }

  it { is_expected.to allow_value("", nil).for(:vDup) }
  it { is_expected.to validate_numericality_of(:vDup) }

  it "should require vDup to be greater than or equal to zero" do
    subject.vDup = -1
    subject.valid?

    expect(subject.errors[:vDup].to_s).to match(/deve ser maior ou igual a 0/)
  end

  it "should require vDup to be less than or equal to 9999999999999.99" do
    subject.vDup = 12_345_678_901_234.56
    subject.valid?

    expect(subject.errors[:vDup].to_s).to match(/deve ser menor ou igual a 9999999999999.99/)
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<nDup>100-1</nDup>"
      xml += "<dVenc>2013-09-25</dVenc>"
      xml += "<vDup>1234567890123.45</vDup>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Bill::Dup.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.nDup).to eql "100-1"
        expect(subject.dVenc).to eql Date.parse("2013-09-25")
        expect(subject.vDup).to eql 1_234_567_890_123.45
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    subject { Nfe::Models::Bill::Dup.new }

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to_not be_present
      end
    end

    describe "#parse!" do
      before { subject.parse!("<root></root>") }

      it "parses xml" do
        expect(subject.nDup).to be_blank
        expect(subject.dVenc).to be_nil
        expect(subject.vDup).to be_blank
      end

      it { is_expected.to be_valid}
    end
  end
end
