require "spec_helper"

describe Nfe::Models::Bill::Fat do
  subject { build(:fat) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to allow_value("", nil).for(:nFat) }
  it { is_expected.to ensure_length_of(:nFat).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:vOrig) }
  it { is_expected.to validate_numericality_of(:vOrig) }

  it "should require vOrig to be greater than or equal to zero" do
    subject.vOrig = -1
    subject.valid?

    expect(subject.errors[:vOrig].to_s).to match(/deve ser maior ou igual a 0/)
  end

  it "should require vOrig to be less than or equal to 9999999999999.99" do
    subject.vOrig = 12_345_678_901_234.56
    subject.valid?

    expect(subject.errors[:vOrig].to_s).to match(/deve ser menor ou igual a 9999999999999.99/)
  end

  it { is_expected.to allow_value("", nil).for(:vDesc) }
  it { is_expected.to validate_numericality_of(:vDesc) }

  it "should require vDesc to be greater than or equal to zero" do
    subject.vDesc = -1
    subject.valid?

    expect(subject.errors[:vDesc].to_s).to match(/deve ser maior ou igual a 0/)
  end

  it "should require vDesc to be less than or equal to 9999999999999.99" do
    subject.vDesc = 12_345_678_901_234.56
    subject.valid?

    expect(subject.errors[:vDesc].to_s).to match(/deve ser menor ou igual a 9999999999999.99/)
  end

  it { is_expected.to allow_value("", nil).for(:vLiq) }
  it { is_expected.to validate_numericality_of(:vLiq) }

  it "should require vLiq to be greater than or equal to zero" do
    subject.vLiq = -1
    subject.valid?

    expect(subject.errors[:vLiq].to_s).to match(/deve ser maior ou igual a 0/)
  end

  it "should require vLiq to be less than or equal to 9999999999999.99" do
    subject.vLiq = 12_345_678_901_234.56
    subject.valid?

    expect(subject.errors[:vLiq].to_s).to match(/deve ser menor ou igual a 9999999999999.99/)
  end

  describe "#present?" do
    subject { Nfe::Models::Bill::Fat.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.nFat = "1" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<nFat>100</nFat>"
      xml += "<vOrig>1234567890123.45</vOrig>"
      xml += "<vDesc>1234567890123.45</vDesc>"
      xml += "<vLiq>1234567890123.45</vLiq>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Bill::Fat.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.nFat).to eql "100"
        expect(subject.vOrig).to eql 1_234_567_890_123.45
        expect(subject.vDesc).to eql 1_234_567_890_123.45
        expect(subject.vLiq).to eql 1_234_567_890_123.45
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    subject { Nfe::Models::Bill::Fat.new }

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to_not be_present
      end
    end

    describe "#parse!" do
      before { subject.parse!("<root></root>") }

      it "parses xml" do
        expect(subject.nFat).to  be_blank
        expect(subject.vOrig).to be_blank
        expect(subject.vDesc).to be_blank
        expect(subject.vLiq).to  be_blank
      end

      it { is_expected.to be_valid }
    end
  end
end
