require "spec_helper"

describe Nfe::Models::Cane::Cana do
  subject { build(:cana) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:safra).is_at_least(4).is_at_most(9) }

  it { is_expected.to ensure_length_of(:ref).is_equal_to(7) }

  it { is_expected.to validate_presence_of :vFor }
  it { is_expected.to validate_numericality_of(:vFor) }

  it { is_expected.to validate_presence_of :vTotDed }
  it { is_expected.to validate_numericality_of(:vTotDed) }

  it { is_expected.to validate_presence_of :vLiqFor }
  it { is_expected.to validate_numericality_of(:vLiqFor) }

  context "when forDia is present and invalid" do
    it "returns an error on fat" do
      forDia = Nfe::Models::Cane::ForDia.new
      subject.forDia << build(:forDia, dia: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/forDia/)
    end
  end

  context "when deduc is present and invalid" do
    it "returns an error on dup" do
      deduc = Nfe::Models::Cane::Deduc.new
      subject.deduc << build(:deduc, vDed: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/deduc/)
    end
  end

  describe "#qTotMes" do
    it "should be greater than or equal to 0" do
      subject.qTotMes = -1
      subject.valid?

      expect(subject.errors[:qTotMes].to_s).to match(/deve ser maior ou igual a 0/)
    end

    it "should be less than or equal to 99999999999.9999999999" do
      subject.qTotMes = "100_000_000_000.0000000000"
      subject.valid?

      expect(subject.errors[:qTotMes].to_s).to match(/deve ser menor ou igual a 99999999999.9999999999/)
    end
  end

  describe "#qTotAnt" do
    it "should be greater than or equal to 0" do
      subject.qTotAnt = -1
      subject.valid?

      expect(subject.errors[:qTotAnt].to_s).to match(/deve ser maior ou igual a 0/)
    end

    it "should be less than or equal to 99999999999.9999999999" do
      subject.qTotAnt = "100_000_000_000.0000000000"
      subject.valid?

      expect(subject.errors[:qTotAnt].to_s).to match(/deve ser menor ou igual a 99999999999.9999999999/)
    end
  end

  describe "#qTotGer" do
    it "should be greater than or equal to 0" do
      subject.qTotGer = -1
      subject.valid?

      expect(subject.errors[:qTotGer].to_s).to match(/deve ser maior ou igual a 0/)
    end

    it "should be less than or equal to 99999999999.9999999999" do
      subject.qTotGer = "100_000_000_000.0000000000"
      subject.valid?

      expect(subject.errors[:qTotGer].to_s).to match(/deve ser menor ou igual a 99999999999.9999999999/)
    end
  end

  describe "#present?" do
    subject { Nfe::Models::Cane::Cana.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.ref = "11/2013" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml = "<safra>2013</safra>"
      xml += "<ref>11/2013</ref>"
      xml += "<forDia dia=\"1\"><qtde>99999999999.9999999999</qtde></forDia>"
      xml += "<forDia dia=\"2\"><qtde>99999999999.9999999999</qtde></forDia>"
      xml += "<forDia dia=\"3\"><qtde>99999999999.9999999999</qtde></forDia>"
      xml += "<qTotMes>99999999999.9999999999</qTotMes>"
      xml += "<qTotAnt>99999999999.9999999999</qTotAnt>"
      xml += "<qTotGer>99999999999.9999999999</qTotGer>"
      xml += "<deduc><xDed>Deduc0</xDed><vDed>12345.67</vDed></deduc>"
      xml += "<deduc><xDed>Deduc1</xDed><vDed>12345.67</vDed></deduc>"
      xml += "<vFor>12345.67</vFor>"
      xml += "<vTotDed>12345.67</vTotDed>"
      xml += "<vLiqFor>12345.67</vLiqFor>"
      xml
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Cane::Cana.new }
      before(:each) { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.safra).to eql "2013"
        expect(subject.ref).to eql "11/2013"

        expect(subject.forDia[0].dia).to eql "1"
        expect(subject.forDia[0].qtde).to eql BigDecimal("99999999999.9999999999")
        expect(subject.forDia[1].dia).to eql "2"
        expect(subject.forDia[1].qtde).to eql BigDecimal("99999999999.9999999999")
        expect(subject.forDia[2].dia).to eql "3"
        expect(subject.forDia[2].qtde).to eql BigDecimal("99999999999.9999999999")

        expect(subject.qTotMes).to eql BigDecimal("99999999999.9999999999")
        expect(subject.qTotAnt).to eql BigDecimal("99999999999.9999999999")
        expect(subject.qTotGer).to eql BigDecimal("99999999999.9999999999")

        expect(subject.deduc[0].xDed).to eql "Deduc0"
        expect(subject.deduc[0].vDed).to eql 12345.67
        expect(subject.deduc[1].xDed).to eql "Deduc1"
        expect(subject.deduc[1].vDed).to eql 12345.67

        expect(subject.vFor).to eql 12345.67
        expect(subject.vTotDed).to eql 12345.67
        expect(subject.vLiqFor).to eql 12345.67
      end

      it { is_expected.to be_valid }
    end
  end
end
