require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Cane::Deduc do
  subject { build(:deduc) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :xDed }
  it { is_expected.to ensure_length_of(:xDed).is_at_least(1).is_at_most(60) }

  it_should_behave_like "a numeric TDec_1302 field", :vDed

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<xDed>#{'A' * 60}</xDed>"
      xml += "<vDed>12345.67</vDed>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Cane::Deduc.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.xDed).to eql "A" * 60
        expect(subject.vDed).to eql 12345.67
      end

      it { is_expected.to be_valid }
    end
  end
end
