require "spec_helper"

describe Nfe::Models::Cane::ForDia do
  subject { build(:forDia) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :dia }
  it { is_expected.to ensure_length_of(:dia).is_at_least(1).is_at_most(2) }
  it { is_expected.to validate_numericality_of(:dia).only_integer }

  describe "#qtde" do
    it "should be greater than or equal to 0" do
      subject.qtde = -1
      subject.valid?

      expect(subject.errors[:qtde].to_s).to match(/deve ser maior ou igual a 0/)
    end

    it "should be less than or equal to 99999999999.9999999999" do
      subject.qtde = "100_000_000_000.0000000000"
      subject.valid?

      expect(subject.errors[:qtde].to_s).to match(/deve ser menor ou igual a 99999999999.9999999999/)
    end
  end

  context "when xml has all tags" do
    let(:xml) { "<qtde>99999999999.9999999999</qtde>" }

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Cane::ForDia.new }
      before { subject.parse!("<cana dia='12'>#{xml}</cana>") }

      it "parses xml" do
        expect(subject.dia).to eql "12"
        expect(subject.qtde).to eql BigDecimal("99_999_999_999.9999999999")
      end

      it { is_expected.to be_valid }
    end
  end
end
