require "spec_helper"

describe Nfe::Models::Compra do
  subject { build(:compra) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to allow_value("", nil).for(:xNEmp) }
  it { is_expected.to ensure_length_of(:xNEmp).is_at_most(22) }

  it { is_expected.to ensure_length_of(:xCont).is_at_most(60) }
  it { is_expected.to ensure_length_of(:xPed).is_at_most(60) }

  describe "#present?" do
    subject { Nfe::Models::Compra.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.xCont = "XXX" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<xNEmp>Nota de Empenho</xNEmp>"
      xml += "<xCont>Informacao do pedido</xCont>"
      xml += "<xPed>Informacao do contrato</xPed>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Compra.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.xNEmp).to eql "Nota de Empenho"
        expect(subject.xCont).to eql "Informacao do pedido"
        expect(subject.xPed).to eql "Informacao do contrato"
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    subject { Nfe::Models::Compra.new }

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to_not be_present
      end
    end

    describe "#parse!" do
      before { subject.parse!("<root></root>") }

      it "parses xml" do
        expect(subject.xNEmp).to be_blank
        expect(subject.xCont).to be_blank
        expect(subject.xPed).to be_blank
      end

      it { is_expected.to be_valid }
    end
  end
end
