require "spec_helper"

describe Nfe::Models::Ender do
  subject { build(:ender) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :xLgr }
  it { is_expected.to ensure_length_of(:xLgr).is_at_least(2).is_at_most(60) }

  it { is_expected.to validate_presence_of :nro }
  it { is_expected.to ensure_length_of(:nro).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:xCpl) }
  it { is_expected.to ensure_length_of(:xCpl).is_at_most(60) }

  it { is_expected.to validate_presence_of :xBairro }
  it { is_expected.to ensure_length_of(:xBairro).is_at_least(2).is_at_most(60) }

  it { is_expected.to validate_presence_of :cMun }
  it { is_expected.to ensure_length_of(:cMun).is_at_most(7) }
  it { is_expected.to validate_numericality_of(:cMun).only_integer }

  it { is_expected.to validate_presence_of :xMun }
  it { is_expected.to ensure_length_of(:xMun).is_at_least(2).is_at_most(60) }

  it { is_expected.to validate_presence_of :uf }
  it { is_expected.to ensure_length_of(:uf).is_equal_to(2) }

  it { is_expected.to validate_presence_of :cep }
  it { is_expected.to ensure_length_of(:cep).is_equal_to(8) }
  it { is_expected.to validate_numericality_of(:cep).only_integer }

  it { is_expected.to allow_value("", nil).for(:cPais) }
  it { is_expected.to ensure_length_of(:cPais).is_equal_to(4) }
  it { is_expected.to validate_numericality_of(:cPais).only_integer }

  it { is_expected.to allow_value("", nil).for(:xPais) }
  it { is_expected.to ensure_length_of(:xPais).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:fone) }
  it { is_expected.to ensure_length_of(:fone).is_at_least(6).is_at_most(14) }
  it { is_expected.to validate_numericality_of(:fone).only_integer }

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<xLgr>Rua Geral</xLgr>"
      xml += "<nro>5</nro>"
      xml += "<xCpl>Apartamento</xCpl>"
      xml += "<xBairro>Centro</xBairro>"
      xml += "<cMun>4203709</cMun>"
      xml += "<xMun>Tijucas</xMun>"
      xml += "<UF>SC</UF>"
      xml += "<CEP>88200000</CEP>"
      xml += "<cPais>1058</cPais>"
      xml += "<xPais>Brasil</xPais>"
      xml += "<fone>4832630000</fone>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { described_class.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.xLgr).to eql "Rua Geral"
        expect(subject.nro).to eql "5"
        expect(subject.xCpl).to eql "Apartamento"
        expect(subject.xBairro).to eql "Centro"
        expect(subject.cMun).to eql "4203709"
        expect(subject.xMun).to eql "Tijucas"
        expect(subject.uf).to eql "SC"
        expect(subject.cep).to eql "88200000"
        expect(subject.cPais).to eql "1058"
        expect(subject.xPais).to eql "Brasil"
        expect(subject.fone).to eql "4832630000"
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<xLgr>Rua Geral</xLgr>"
      xml += "<nro>5</nro>"
      xml += "<xBairro>Centro</xBairro>"
      xml += "<cMun>4203709</cMun>"
      xml += "<xMun>Tijucas</xMun>"
      xml += "<UF>SC</UF>"
      xml += "<CEP>88200000</CEP>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:ender, xCpl: "", cPais: "", xPais: "", fone: "")
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { described_class.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.xLgr).to eql "Rua Geral"
        expect(subject.nro).to eql "5"
        expect(subject.xCpl).to eql nil
        expect(subject.xBairro).to eql "Centro"
        expect(subject.cMun).to eql "4203709"
        expect(subject.xMun).to eql "Tijucas"
        expect(subject.uf).to eql "SC"
        expect(subject.cep).to eql "88200000"
        expect(subject.cPais).to be_nil
        expect(subject.xPais).to be_nil
        expect(subject.fone).to be_nil
      end

      it { is_expected.to be_valid }
    end
  end
end
