require "spec_helper"

describe Nfe::Models::Evento do
  subject { build(:evento) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:versao).is_equal_to(4) }

  it { is_expected.to ensure_length_of(:uf).is_equal_to(2) }

  it { is_expected.to ensure_length_of(:tpEvento).is_equal_to(6) }

  it { is_expected.to ensure_length_of(:nSeqEvento).is_at_least(1).is_at_most(2) }

  it { is_expected.to ensure_length_of(:chNFe).is_equal_to(44) }

  context "when detEvento is invalid" do
    it "returns an error on detEvento" do
      subject.detEvento = build(:detEventoCancel, nProt: nil)
      subject.valid?

      expect(subject.errors.to_json).to match(/detEvento/)
    end
  end

  let(:xml) do
    xml =  "<evento xmlns=\"http://www.portalfiscal.inf.br/nfe\" versao=\"1.00\">"
    xml +=   "<infEvento Id=\"ID1101111234567891234567891234567891234567891234567801\">"
    xml +=     "<cOrgao>43</cOrgao>"
    xml +=     "<tpAmb>1</tpAmb>"
    xml +=     "<CNPJ>01603301000504</CNPJ>"
    xml +=     "<CPF>11144477735</CPF>"
    xml +=     "<chNFe>12345678912345678912345678912345678912345678</chNFe>"
    xml +=     "<dhEvento>2013-04-03T10:08:42-03:00</dhEvento>"
    xml +=     "<tpEvento>110111</tpEvento>"
    xml +=     "<nSeqEvento>1</nSeqEvento>"
    xml +=     "<verEvento>1.00</verEvento>"
    xml +=     build(:detEventoCancel).to_xml
    xml +=   "</infEvento>"
    xml +=   "<Signature xmlns=\"http://www.w3.org/2000/09/xmldsig#\">#{build(:signature).value}</Signature>"
    xml += "</evento>"
  end

  describe "#to_xml" do
    it "returns a formatted xml" do
      expect(subject.to_xml).to eql xml
    end
  end
end
