require "spec_helper"

describe Nfe::Models::Exporta do
  subject { build(:exporta) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :ufEmbarq }
  it { is_expected.to ensure_length_of(:ufEmbarq).is_equal_to(2) }

  it { is_expected.to validate_presence_of :xLocEmbarq }
  it { is_expected.to ensure_length_of(:xLocEmbarq).is_at_least(1).is_at_most(60) }

  describe "#present?" do
    subject { Nfe::Models::Exporta.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.ufEmbarq = "SC" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<UFEmbarq>SC</UFEmbarq>"
      xml += "<xLocEmbarq>Porto de São Francisco do Sul</xLocEmbarq>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Exporta.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.ufEmbarq).to eql "SC"
        expect(subject.xLocEmbarq).to eql "Porto de São Francisco do Sul"
      end

      it { is_expected.to be_valid }
    end
  end
end
