require "spec_helper"

describe Nfe::Models::Information::InfAdic do
  subject { build(:infAdic) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to allow_value("", nil).for(:infAdFisco) }
  it { is_expected.to ensure_length_of(:infAdFisco).is_at_most(2000) }

  it { is_expected.to allow_value("", nil).for(:infCpl) }
  it { is_expected.to ensure_length_of(:infCpl).is_at_most(5000) }

  context "when obsFisco is present and invalid" do
    it "returns an error on dup" do
      subject.obsFisco << build(:obsContFisco, xTexto: "A"*61 )
      subject.valid?

      expect(subject.errors.to_json).to match(/obsFisco/)
    end
  end

  context "when obsCont is present and invalid" do
    it "returns an error on dup" do
      subject.procRef << build(:procRef, indProc: 50 )
      subject.valid?

      expect(subject.errors.to_json).to match(/procRef/)
    end
  end

  context "when obsCont is present and invalid" do
    it "returns an error on dup" do
      subject.obsCont << build(:obsContFisco, xTexto: "A"*61 )
      subject.valid?

      expect(subject.errors.to_json).to match(/obsCont/)
    end
  end

  describe "#obsCont" do
    it "should be less than or equal to 10" do
      11.times { subject.obsCont << build(:obsContFisco) }
      subject.valid?
      expect(subject.errors[:obsCont].to_s).to match(/deve ser menor ou igual a 10/)
    end
  end

  describe "#obsFisco" do
    it "should be less than or equal to 10" do
      11.times { subject.obsFisco << build(:obsContFisco) }
      subject.valid?
      expect(subject.errors[:obsFisco].to_s).to match(/deve ser menor ou igual a 10/)
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<infAdFisco>AAAAAAAAAA</infAdFisco>"
      xml += "<infCpl>OOOOOOOOOO</infCpl>"
      xml += "<obsCont xCampo=\"Campo0\"><xTexto>Texto0</xTexto></obsCont>"
      xml += "<obsCont xCampo=\"Campo1\"><xTexto>Texto1</xTexto></obsCont>"
      xml += "<obsFisco xCampo=\"Campo0\"><xTexto>Texto0</xTexto></obsFisco>"
      xml += "<obsFisco xCampo=\"Campo1\"><xTexto>Texto1</xTexto></obsFisco>"
      xml += "<procRef><nProc>Texto0</nProc><indProc>0</indProc></procRef>"
      xml += "<procRef><nProc>Texto1</nProc><indProc>1</indProc></procRef>"
      xml
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Information::InfAdic.new }
      before(:each) { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.infAdFisco).to eql "AAAAAAAAAA"
        expect(subject.infCpl).to eql "OOOOOOOOOO"
        expect(subject.obsCont[0].xCampo).to eql "Campo0"
        expect(subject.obsCont[0].xTexto).to eql "Texto0"
        expect(subject.obsCont[1].xCampo).to eql "Campo1"
        expect(subject.obsCont[1].xTexto).to eql "Texto1"
        expect(subject.obsFisco[0].xCampo).to eql "Campo0"
        expect(subject.obsFisco[0].xTexto).to eql "Texto0"
        expect(subject.obsFisco[1].xCampo).to eql "Campo1"
        expect(subject.obsFisco[1].xTexto).to eql "Texto1"
        expect(subject.procRef[0].nProc).to eql "Texto0"
        expect(subject.procRef[0].indProc).to eql "sefaz"
        expect(subject.procRef[1].nProc).to eql "Texto1"
        expect(subject.procRef[1].indProc).to eql "justica_federal"
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    subject { Nfe::Models::Information::InfAdic.new }

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to_not be_present
      end
    end

    describe "#parse!" do
      before { subject.parse!("<root></root>") }

      it "parses xml" do
        expect(subject.infAdFisco).to be_blank
        expect(subject.infCpl).to be_blank
        expect(subject.obsCont).to be_blank
        expect(subject.obsFisco).to be_blank
        expect(subject.procRef).to be_blank
      end

      it { is_expected.to be_valid }
    end
  end
end
