require "spec_helper"
require_relative "../shared_examples_for_tdec_1104"
require_relative "../shared_examples_for_tdec_1204"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Itens::CIDE do
  subject { build(:cide) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it_should_behave_like "a numeric TDec_1204 field", :qBCProd
  it_should_behave_like "a numeric TDec_1104 field", :vAliqProd
  it_should_behave_like "a numeric TDec_1302 field", :vCIDE

  describe "#present?" do
    subject { Nfe::Models::Itens::CIDE.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.qBCProd = 1 }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<qBCProd>98765.4321</qBCProd>"
      xml += "<vAliqProd>123.4567</vAliqProd>"
      xml += "<vCIDE>12345.89</vCIDE>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Itens::CIDE.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.qBCProd).to eql 98765.4321
        expect(subject.vAliqProd).to eql 123.4567
        expect(subject.vCIDE).to eql 12345.89
      end

      it { is_expected.to be_valid }
    end
  end
end
