require "spec_helper"

describe Nfe::Models::Itens::DI do
  subject { build(:di) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:nDI).is_at_least(1).is_at_most(12) }
  it { is_expected.to ensure_length_of(:xLocDesemb).is_at_least(1).is_at_most(60) }
  it { is_expected.to ensure_length_of(:ufDesemb).is_equal_to(2) }
  it { is_expected.to ensure_length_of(:cExportador).is_at_least(1).is_at_most(60) }

  context "when adi is present and invalid" do
    it "returns an error on adi" do
      subject.adi << build(:adi, nAdicao: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/adi/)
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<nDI>12X3ABC4</nDI>"
      xml += "<dDI>2013-11-29</dDI>"
      xml += "<xLocDesemb>Porto de Itajaí</xLocDesemb>"
      xml += "<UFDesemb>SC</UFDesemb>"
      xml += "<dDesemb>2013-11-28</dDesemb>"
      xml += "<cExportador>95214A2311</cExportador>"
      xml += "<adi><nAdicao>1</nAdicao><nSeqAdic>1</nSeqAdic><cFabricante>2345678</cFabricante><vDescDI>10.30</vDescDI></adi>"
      xml += "<adi><nAdicao>1</nAdicao><nSeqAdic>2</nSeqAdic><cFabricante>2345678</cFabricante><vDescDI>10.30</vDescDI></adi>"
      xml
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Itens::DI.new }
      before(:each) { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.nDI).to eql "12X3ABC4"
        expect(subject.dDI).to eql Date.parse("2013-11-29")
        expect(subject.xLocDesemb).to eql "Porto de Itajaí"
        expect(subject.ufDesemb).to eql "SC"
        expect(subject.dDesemb).to eql Date.parse("2013-11-28")
        expect(subject.cExportador).to eql "95214A2311"
        expect(subject.adi[0].nAdicao).to eql "1"
        expect(subject.adi[0].nSeqAdic).to eql "1"
        expect(subject.adi[0].cFabricante).to eql "2345678"
        expect(subject.adi[0].vDescDI).to eql 10.30
        expect(subject.adi[1].nAdicao).to eql "1"
        expect(subject.adi[1].nSeqAdic).to eql "2"
        expect(subject.adi[1].cFabricante).to eql "2345678"
        expect(subject.adi[1].vDescDI).to eql 10.30
      end

      it { is_expected.to be_valid }
    end
  end
end
