require "spec_helper"

describe Nfe::Models::Itens::Adi do
  subject { build(:adi) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :nAdicao }
  it { is_expected.to ensure_length_of(:nAdicao).is_at_most(3) }

  it { is_expected.to validate_presence_of :nSeqAdic }
  it { is_expected.to ensure_length_of(:nSeqAdic).is_at_most(3) }

  it { is_expected.to validate_presence_of :cFabricante }
  it { is_expected.to ensure_length_of(:cFabricante).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:vDescDI) }
  it { is_expected.to validate_numericality_of(:vDescDI) }

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<nAdicao>1</nAdicao>"
      xml += "<nSeqAdic>1</nSeqAdic>"
      xml += "<cFabricante>2345678</cFabricante>"
      xml += "<vDescDI>10.30</vDescDI>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Itens::Adi.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.nAdicao).to eql "1"
        expect(subject.nSeqAdic).to eql "1"
        expect(subject.cFabricante).to eql "2345678"
        expect(subject.vDescDI).to eql 10.30
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<nAdicao>1</nAdicao>"
      xml += "<nSeqAdic>1</nSeqAdic>"
      xml += "<cFabricante>2345678</cFabricante>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:adi, vDescDI: "")
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Itens::Adi.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.nAdicao).to eql "1"
        expect(subject.nSeqAdic).to eql "1"
        expect(subject.cFabricante).to eql "2345678"
        expect(subject.vDescDI).to be_nil
      end

      it { is_expected.to be_valid }
    end
  end
end
