require "spec_helper"
require_relative "../shared_examples_for_tdec_1204"

describe Nfe::Models::Itens::Comb do
  subject { build(:comb) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:cProdANP).is_equal_to(9) }

  it { is_expected.to allow_value("", nil).for(:codif) }
  it { is_expected.to ensure_length_of(:codif).is_at_least(0).is_at_most(21) }
  it { is_expected.to validate_numericality_of(:codif).only_integer }

  it_should_behave_like "a numeric TDec_1204Opc field", :qTemp

  it { is_expected.to ensure_length_of(:ufCons).is_equal_to(2) }

  describe "#present?" do
    subject { Nfe::Models::Itens::Comb.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.qTemp = 123  }

      it { is_expected.to be_present }
    end
  end

  context "when cide is present and invalid" do
    it "returns an error on cide" do
      subject.cide = build(:cide, qBCProd: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/cide/)
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<cProdANP>110203073</cProdANP>"
      xml += "<CODIF>12345</CODIF>"
      xml += "<qTemp>123456.1234</qTemp>"
      xml += "<UFCons>SC</UFCons>"
      xml += "<CIDE>#{build(:cide).to_xml}</CIDE>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Itens::Comb.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cProdANP).to eql "110203073"
        expect(subject.codif).to eql "12345"
        expect(subject.qTemp).to eql 123_456.1234
        expect(subject.ufCons).to eql "SC"
        expect(subject.cide.to_json).to eql build(:cide).to_json
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<cProdANP>110203073</cProdANP>"
      xml += "<UFCons>SC</UFCons>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:comb, codif: nil, qTemp: nil, cide: nil)
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Itens::Comb.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cProdANP).to eql "110203073"
        expect(subject.codif).to be_nil
        expect(subject.qTemp).to be_nil
        expect(subject.ufCons).to eql "SC"
        expect(subject.cide).to_not be_present
      end

      it { is_expected.to be_valid }
    end
  end
end
