require "spec_helper"
require_relative "../shared_examples_for_tdec_0803"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Itens::Med do
  subject { build(:med) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of(:nLote) }
  it { is_expected.to ensure_length_of(:nLote).is_at_most(20) }

  it_should_behave_like "a numeric TDec_0803 field", :qLote

  it { is_expected.to validate_presence_of(:dFab) }
  it { is_expected.to validate_presence_of(:dVal) }

  it_should_behave_like "a numeric TDec_1302 field", :vPMC

  context "when xml has all tags" do
    let(:xml) do
      xml = "<nLote>123456789</nLote>"
      xml += "<qLote>9811.658</qLote>"
      xml += "<dFab>2013-11-29</dFab>"
      xml += "<dVal>2013-11-30</dVal>"
      xml += "<vPMC>456.99</vPMC>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Itens::Med.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.nLote).to eql "123456789"
        expect(subject.qLote).to eql 9811.658
        expect(subject.dFab).to eql Date.parse("2013-11-29")
        expect(subject.dVal).to eql Date.parse("2013-11-30")
        expect(subject.vPMC).to eql 456.99
      end

      it { is_expected.to be_valid }
    end
  end
end
