require "spec_helper"

describe Nfe::Models::Itens::VeicProd do
  subject { build(:veicProd) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of(:tpOp) }
  it { is_expected.to validate_inclusion_of(:tpOp).in_array(["0", "1", "2", "3"]) }

  it { is_expected.to validate_presence_of(:chassi) }
  it { is_expected.to ensure_length_of(:chassi).is_equal_to(17) }

  it { is_expected.to validate_presence_of(:cCor) }
  it { is_expected.to ensure_length_of(:cCor).is_at_most(4) }

  it { is_expected.to validate_presence_of(:xCor) }
  it { is_expected.to ensure_length_of(:xCor).is_at_most(40) }

  it { is_expected.to validate_presence_of(:pot) }
  it { is_expected.to ensure_length_of(:pot).is_at_most(4) }

  it { is_expected.to validate_presence_of(:cilin) }
  it { is_expected.to ensure_length_of(:cilin).is_equal_to(4) }

  it { is_expected.to validate_presence_of(:pesoL) }
  it { is_expected.to ensure_length_of(:pesoL).is_at_most(9) }

  it { is_expected.to validate_presence_of(:pesoB) }
  it { is_expected.to ensure_length_of(:pesoB).is_at_most(9) }

  it { is_expected.to validate_presence_of(:nSerie) }
  it { is_expected.to ensure_length_of(:nSerie).is_at_most(9) }

  it { is_expected.to validate_presence_of(:tpComb) }
  it { is_expected.to ensure_length_of(:tpComb).is_at_most(2) }
  it { is_expected.to validate_numericality_of(:tpComb).only_integer }

  it { is_expected.to validate_presence_of(:nMotor) }
  it { is_expected.to ensure_length_of(:nMotor).is_at_most(21) }

  it { is_expected.to validate_presence_of(:cmt) }
  it { is_expected.to ensure_length_of(:cmt).is_at_most(9) }

  it { is_expected.to validate_presence_of(:dist) }
  it { is_expected.to ensure_length_of(:dist).is_at_most(4) }
  it { is_expected.to validate_numericality_of(:dist).only_integer }

  it { is_expected.to validate_presence_of(:anoMod) }
  it { is_expected.to ensure_length_of(:anoMod).is_equal_to(4) }
  it { is_expected.to validate_numericality_of(:anoMod).only_integer }

  it { is_expected.to validate_presence_of(:anoFab) }
  it { is_expected.to ensure_length_of(:anoFab).is_equal_to(4) }
  it { is_expected.to validate_numericality_of(:anoFab).only_integer }

  it { is_expected.to validate_presence_of(:tpPint) }
  it { is_expected.to ensure_length_of(:tpPint).is_equal_to(1) }

  it { is_expected.to validate_presence_of(:tpVeic) }
  it { is_expected.to ensure_length_of(:tpVeic).is_at_most(2) }
  it { is_expected.to validate_numericality_of(:tpVeic).only_integer }

  it { is_expected.to validate_presence_of(:espVeic) }
  it { is_expected.to ensure_length_of(:espVeic).is_equal_to(1) }
  it { is_expected.to validate_numericality_of(:espVeic).only_integer }

  it { is_expected.to validate_presence_of(:vin) }
  it { is_expected.to validate_inclusion_of(:vin).in_array(["R", "N"]) }

  it { is_expected.to validate_presence_of(:condVeic) }
  it { is_expected.to validate_inclusion_of(:condVeic).in_array(["1", "2", "3"]) }

  it { is_expected.to validate_presence_of(:cMod) }
  it { is_expected.to ensure_length_of(:cMod).is_at_most(6) }
  it { is_expected.to validate_numericality_of(:cMod).only_integer }

  it { is_expected.to validate_presence_of(:cCorDENATRAN) }
  it { is_expected.to ensure_length_of(:cCorDENATRAN).is_at_most(2) }
  it { is_expected.to validate_numericality_of(:cCorDENATRAN).only_integer }

  it { is_expected.to validate_presence_of(:lota) }
  it { is_expected.to ensure_length_of(:lota).is_at_most(3) }
  it { is_expected.to validate_numericality_of(:lota).only_integer }

  it { is_expected.to validate_presence_of(:tpRest) }
  it { is_expected.to validate_inclusion_of(:tpRest).in_array(["0", "1", "2", "3", "4", "9"]) }

  describe "#present?" do
    subject { Nfe::Models::Itens::VeicProd.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.tpOp = "0" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<tpOp>2</tpOp>"
      xml += "<chassi>ABC45678901234567</chassi>"
      xml += "<cCor>100A</cCor>"
      xml += "<xCor>Descrição da Cor</xCor>"
      xml += "<pot>2000</pot>"
      xml += "<cilin>5000</cilin>"
      xml += "<pesoL>3.4567</pesoL>"
      xml += "<pesoB>12.3456</pesoB>"
      xml += "<nSerie>123456789</nSerie>"
      xml += "<tpComb>01</tpComb>"
      xml += "<nMotor>12345678901234567890</nMotor>"
      xml += "<CMT>0.1234</CMT>"
      xml += "<dist>123</dist>"
      xml += "<anoMod>2013</anoMod>"
      xml += "<anoFab>2012</anoFab>"
      xml += "<tpPint>A</tpPint>"
      xml += "<tpVeic>06</tpVeic>"
      xml += "<espVeic>1</espVeic>"
      xml += "<VIN>R</VIN>"
      xml += "<condVeic>1</condVeic>"
      xml += "<cMod>12345</cMod>"
      xml += "<cCorDENATRAN>04</cCorDENATRAN>"
      xml += "<lota>20</lota>"
      xml += "<tpRest>9</tpRest>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Itens::VeicProd.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.tpOp).to eql "2"
        expect(subject.chassi).to eql "ABC45678901234567"
        expect(subject.cCor).to eql "100A"
        expect(subject.xCor).to eql "Descrição da Cor"
        expect(subject.pot).to eql "2000"
        expect(subject.cilin).to eql "5000"
        expect(subject.pesoL).to eql "3.4567"
        expect(subject.pesoB).to eql "12.3456"
        expect(subject.nSerie).to eql "123456789"
        expect(subject.tpComb).to eql "01"
        expect(subject.nMotor).to eql "12345678901234567890"
        expect(subject.cmt).to eql "0.1234"
        expect(subject.dist).to eql "123"
        expect(subject.anoMod).to eql "2013"
        expect(subject.anoFab).to eql "2012"
        expect(subject.tpPint).to eql "A"
        expect(subject.tpVeic).to eql "06"
        expect(subject.espVeic).to eql "1"
        expect(subject.vin).to eql "R"
        expect(subject.condVeic).to eql "1"
        expect(subject.cMod).to eql "12345"
        expect(subject.cCorDENATRAN).to eql "04"
        expect(subject.lota).to eql "20"
        expect(subject.tpRest).to eql "9"
      end

      it { is_expected.to be_valid }
    end
  end
end
