require "spec_helper"

describe Nfe::Models::Referenced::NFref do
  subject { build(:NFref) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:refNFe).is_equal_to(44) }

  it { is_expected.to ensure_length_of(:refCTe).is_equal_to(44) }

  context "when refNF is present and invalid" do
    it "returns an error on refNF" do
      subject.refNF = build(:refNF, uf: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/refNF/)
    end
  end

  context "when refNFP is present and invalid" do
    it "returns an error on refNFP" do
      subject.refNFP = build(:refNFP, uf: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/refNFP/)
    end
  end

  context "when refECF is present and invalid" do
    it "returns an error on refECF" do
      subject.refECF = build(:refECF, mod: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/refECF/)
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<refNFe>#{"A" * 44}</refNFe>"
      xml += "<refNF>#{build(:refNF).to_xml}</refNF>"
      xml += "<refNFP>#{build(:refNFP).to_xml}</refNFP>"
      xml += "<refCTe>#{"B" * 44}</refCTe>"
      xml += "<refECF>#{build(:refECF).to_xml}</refECF>"
      xml
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Referenced::NFref.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.refNFe).to eql "A" * 44
        expect(subject.refCTe).to eql "B" * 44

        expect(subject.refNF.to_xml).to eql build(:refNF).to_xml
        expect(subject.refNFP.to_xml).to eql build(:refNFP).to_xml
        expect(subject.refECF.to_xml).to eql build(:refECF).to_xml
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have any optional tags" do
    subject { Nfe::Models::Referenced::NFref.new }

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to_not be_present
      end
    end

    describe "#parse!" do
      before { subject.parse!("<root></root>") }

      it "parses xml" do
        expect(subject.refNFe).to_not be_present
        expect(subject.refCTe).to_not be_present
        expect(subject.refNF).to_not be_present
        expect(subject.refNFP).to_not be_present
        expect(subject.refECF).to_not be_present
      end

      it { is_expected.not_to be_valid }
    end

    it "adds error in model" do
      subject.valid?
      expect(subject.errors.messages[:NFref]).to eql [t("errors.messages.empty")]
    end
  end
end
