require "spec_helper"

describe Nfe::Models::Referenced::RefECF do
  subject { build(:refECF) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of(:mod) }
  it { is_expected.to validate_inclusion_of(:mod).in_array(["2B", "2C", "2D"]) }

  it { is_expected.to validate_presence_of(:nECF) }
  it { is_expected.to ensure_length_of(:nECF).is_equal_to(3) }

  it { is_expected.to validate_presence_of(:nCOO) }
  it { is_expected.to ensure_length_of(:nCOO).is_equal_to(6) }

  describe "#present?" do
    subject { Nfe::Models::Referenced::RefECF.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.mod = "2B" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<mod>2B</mod>"
      xml += "<nECF>001</nECF>"
      xml += "<nCOO>000001</nCOO>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Referenced::RefECF.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.mod).to eql "2B"
        expect(subject.nECF).to eql "001"
        expect(subject.nCOO).to eql "000001"
      end

      it { is_expected.to be_valid }
    end
  end
end
