require "spec_helper"

describe Nfe::Models::Referenced::RefNF do
  subject { build(:refNF) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :cUF }
  it { is_expected.to ensure_length_of(:cUF).is_equal_to(2) }

  it { is_expected.to validate_presence_of :aamm }
  it { is_expected.to ensure_length_of(:aamm).is_equal_to(4) }

  it { is_expected.to validate_presence_of :mod }
  it { is_expected.to ensure_length_of(:mod).is_equal_to(2) }

  it { is_expected.to validate_presence_of :serie }
  it { is_expected.to ensure_length_of(:serie).is_at_least(1).is_at_most(3) }

  it { is_expected.to validate_presence_of :nNF }
  it { is_expected.to ensure_length_of(:nNF).is_at_least(1).is_at_most(9) }

  describe "#CNPJ" do
    it { is_expected.to validate_presence_of :cnpj }

    it { is_expected.to ensure_length_of(:cnpj).is_equal_to(14) }

    context "when number is invalid" do
      before { subject.cnpj = "04001155000103" }
      it { is_expected.not_to be_valid }
    end
  end

  describe "#present?" do
    subject { Nfe::Models::Referenced::RefNF.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.uf = "SC" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<cUF>12</cUF>"
      xml += "<AAMM>1301</AAMM>"
      xml += "<CNPJ>01144012000152</CNPJ>"
      xml += "<mod>01</mod>"
      xml += "<serie>1</serie>"
      xml += "<nNF>999999999</nNF>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Referenced::RefNF.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.uf).to eql "AC"
        expect(subject.aamm).to eql "1301"
        expect(subject.cnpj ).to eql "01144012000152"
        expect(subject.mod).to eql "01"
        expect(subject.serie).to eql "1"
        expect(subject.nNF).to eql "999999999"
      end

      it { is_expected.to be_valid }
    end
  end
end
