require "spec_helper"

describe Nfe::Models::Signature do
  subject { build(:signature) }

  it { is_expected.to validate_presence_of(:id) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  describe "#value" do
    let(:xml) do
      xml  = "<SignedInfo>"
      xml +=   "<CanonicalizationMethod Algorithm='http://www.w3.org/TR/2001/REC-xml-c14n-20010315'/>"
      xml +=   "<SignatureMethod Algorithm='http://www.w3.org/2000/09/xmldsig#rsa-sha1'/>"
      xml +=   "<Reference URI='#AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'>"
      xml +=     "<Transforms>"
      xml +=       "<Transform Algorithm='http://www.w3.org/2000/09/xmldsig#enveloped-signature'/>"
      xml +=       "<Transform Algorithm='http://www.w3.org/TR/2001/REC-xml-c14n-20010315'/>"
      xml +=     "</Transforms>"
      xml +=     "<DigestMethod Algorithm='http://www.w3.org/2000/09/xmldsig#sha1'/>"
      xml +=     "<DigestValue></DigestValue>"
      xml +=   "</Reference>"
      xml += "</SignedInfo>"
      xml += "<SignatureValue></SignatureValue>"
      xml += "<KeyInfo>"
      xml +=   "<X509Data>"
      xml +=     "<X509Certificate></X509Certificate>"
      xml +=   "</X509Data>"
      xml += "</KeyInfo>"
    end

    context "when has a sign" do
      it "returns it value" do
        expect(subject.value).to eql "Signature"
      end
    end

    context "when doesn't have a sign" do
      subject { Nfe::Models::Signature.new("A" * 47) }

      it "returns a unsigned signature" do
        expect(subject.value).to eql xml
      end
    end
  end

  describe "#parse!" do
    before { subject.parse!("<root>Signature</root>") }

    it "parses xml values to itself" do
      expect(subject.value).to eql build(:signature).value
    end
  end
end
