require "spec_helper"
require_relative "../shared_examples_for_tdec_0302"
require_relative "../shared_examples_for_tdec_1302"
require_relative "shared_examples_for_ICMS203"

describe Nfe::Models::Taxes::ICMS203 do
  subject { build(:icms203) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_inclusion_of(:orig).in_array([0, 1, 2]) }
  it { is_expected.to validate_presence_of :csosn }
  it { is_expected.to respond_to "csosn" }
  it { is_expected.to validate_inclusion_of(:modBCST).in_array([0, 1, 2, 3, 4, 5]) }

  it_should_behave_like "a numeric TDec_0302Opc field", :pMVAST
  it_should_behave_like "a numeric TDec_0302Opc field", :pRedBCST
  it_should_behave_like "a numeric TDec_1302 field", :vBCST
  it_should_behave_like "a numeric TDec_0302 field", :pICMSST
  it_should_behave_like "a numeric TDec_1302 field", :vICMSST

  it_should_behave_like "a ICMS203", :icms203, "", Nfe::Models::Taxes::ICMS203.new

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<orig>2</orig>"
      xml += "<CSOSN>203</CSOSN>"
      xml += "<modBCST>5</modBCST>"
      xml += "<vBCST>91234.56</vBCST>"
      xml += "<pICMSST>678.91</pICMSST>"
      xml += "<vICMSST>12345.67</vICMSST>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:icms203, pMVAST: nil, pRedBCST: nil)
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::ICMS203.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.orig).to eql 2
        expect(subject.csosn).to eql "203"
        expect(subject.modBCST).to eql 5
        expect(subject.pMVAST).to be_nil
        expect(subject.pRedBCST).to be_nil
        expect(subject.vBCST).to eql 91234.56
        expect(subject.pICMSST).to eql 678.91
        expect(subject.vICMSST).to eql 12345.67
      end

      it { is_expected.to be_valid }
    end
  end
end
