require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"
require_relative "shared_examples_for_ICMS500"

describe Nfe::Models::Taxes::ICMS500 do
  subject { build(:icms500) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_inclusion_of(:orig).in_array([0, 1, 2]) }
  it { is_expected.to validate_presence_of :csosn }
  it { is_expected.to respond_to "csosn" }

  it_should_behave_like "a numeric TDec_1302Opc field", :vBCSTRet
  it_should_behave_like "a numeric TDec_1302Opc field", :vICMSSTRet

  it_should_behave_like "a ICMS500", :icms500, "", Nfe::Models::Taxes::ICMS500.new

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<orig>2</orig>"
      xml += "<CSOSN>500</CSOSN>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:icms500, vBCSTRet: nil, vICMSSTRet: nil)
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::ICMS500.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.orig).to eql 2
        expect(subject.csosn).to eql "500"
      end

      it { is_expected.to be_valid }
    end
  end
end
