require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"
require_relative "shared_examples_for_ICMS50"

describe Nfe::Models::Taxes::ICMS50 do
  subject { build(:icms50) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :orig }
  it { is_expected.to validate_numericality_of(:orig) }
  it { is_expected.to validate_inclusion_of(:orig).in_array([0, 1, 2]) }
  it { is_expected.to validate_presence_of(:cst) }

  it_should_behave_like "a numeric TDec_1302Opc field", :vICMSDeson

  it { is_expected.to validate_inclusion_of(:motDesICMS).
      in_array([1, 2, 3, 4, 5, 6, 7, 8, 9]).allow_blank(true) }

  it_should_behave_like "a ICMS50", :icms50, "", Nfe::Models::Taxes::ICMS50.new

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<orig>0</orig>"
      xml += "<CST>50</CST>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:icms50, vICMSDeson: nil, motDesICMS: nil)
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::ICMS50.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.orig).to eql 0
        expect(subject.cst).to eql "50"
        expect(subject.vICMSDeson).to be_nil
        expect(subject.motDesICMS).to be_nil
      end

      it { is_expected.to be_valid }
    end
  end
end
