require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"
require_relative "../shared_examples_for_tdec_0302"
require_relative "shared_examples_for_ICMS51"

describe Nfe::Models::Taxes::ICMS51 do
  subject { build(:icms51) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_inclusion_of(:orig).in_array([0, 1, 2]) }
  it { is_expected.to validate_presence_of(:cst) }
  it { is_expected.to respond_to "cst" }
  it { is_expected.to validate_inclusion_of(:modBC).in_array([0, 1, 2, 3]).allow_blank(true) }

  it_should_behave_like "a numeric TDec_0302Opc field", :pRedBC
  it_should_behave_like "a numeric TDec_1302Opc field", :vBC
  it_should_behave_like "a numeric TDec_0302Opc field", :pICMS
  it_should_behave_like "a numeric TDec_1302Opc field", :vICMS

  it_should_behave_like "a ICMS51", :icms51, "", Nfe::Models::Taxes::ICMS51.new

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml = "<orig>2</orig>"
      xml += "<CST>51</CST>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:icms51, modBC: nil, pRedBC: nil, vBC: nil, pICMS: nil, vICMS: nil)
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::ICMS51.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.orig).to eql 2
        expect(subject.cst).to eql "51"
        expect(subject.modBC).to be_nil
        expect(subject.vBC).to be_nil
        expect(subject.pRedBC).to be_nil
        expect(subject.pICMS).to be_nil
        expect(subject.vICMS).to be_nil
      end

      it { is_expected.to be_valid }
    end
  end
end
