require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"
require_relative "shared_examples_for_ICMS60"

describe Nfe::Models::Taxes::ICMS60 do
  subject { build(:icms60) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :orig }
  it { is_expected.to validate_numericality_of(:orig) }
  it { is_expected.to validate_inclusion_of(:orig).in_array([0, 1, 2]) }
  it { is_expected.to validate_presence_of :cst }
  it { is_expected.to respond_to "cst" }


  it_should_behave_like "a numeric TDec_1302Opc field", :vBCSTRet
  it_should_behave_like "a numeric TDec_1302Opc field", :vICMSSTRet

  it_should_behave_like "a ICMS60", :icms60, "", Nfe::Models::Taxes::ICMS60.new

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<orig>2</orig>"
      xml += "<CST>60</CST>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:icms60, vBCSTRet: nil, vICMSSTRet: nil)
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::ICMS60.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.orig).to eql 2
        expect(subject.cst).to eql "60"
        expect(subject.vBCSTRet).to be_nil
        expect(subject.vICMSSTRet).to be_nil
      end

      it { is_expected.to be_valid }
    end
  end

end
