require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"
require_relative "../shared_examples_for_tdec_0302"
require_relative "shared_examples_for_ICMS900"

describe Nfe::Models::Taxes::ICMS900 do
  subject { build(:icms900) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_inclusion_of(:orig).in_array([0, 1, 2]) }
  it { is_expected.to validate_presence_of :csosn }
  it { is_expected.to respond_to "csosn" }
  it { is_expected.to validate_inclusion_of(:modBC).in_array([0, 1, 2, 3]) }

  it_should_behave_like "a numeric TDec_1302 field", :vBC
  it_should_behave_like "a numeric TDec_0302Opc field", :pRedBC
  it_should_behave_like "a numeric TDec_0302 field", :pICMS
  it_should_behave_like "a numeric TDec_1302 field", :vICMS
  it_should_behave_like "a numeric TDec_0302Opc field", :pMVAST
  it_should_behave_like "a numeric TDec_0302Opc field", :pRedBCST
  it_should_behave_like "a numeric TDec_1302 field", :vBCST
  it_should_behave_like "a numeric TDec_0302 field", :pICMSST
  it_should_behave_like "a numeric TDec_1302 field", :vICMSST
  it_should_behave_like "a numeric TDec_0302 field", :pCredSN
  it_should_behave_like "a numeric TDec_1302 field", :vCredICMSSN

  it { is_expected.to validate_inclusion_of(:modBCST).in_array([0, 1, 2, 3, 4, 5]) }

  it_should_behave_like "a ICMS900", :icms900, "", Nfe::Models::Taxes::ICMS900.new

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<orig>2</orig>"
      xml += "<CSOSN>900</CSOSN>"
      xml += "<modBC>3</modBC>"
      xml += "<vBC>12345.67</vBC>"
      xml += "<pICMS>234.56</pICMS>"
      xml += "<vICMS>23456.78</vICMS>"
      xml += "<modBCST>5</modBCST>"
      xml += "<vBCST>91234.56</vBCST>"
      xml += "<pICMSST>678.91</pICMSST>"
      xml += "<vICMSST>12345.67</vICMSST>"
      xml += "<pCredSN>234.56</pCredSN>"
      xml += "<vCredICMSSN>45678.90</vCredICMSSN>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:icms900, pRedBC: nil, pMVAST: nil, pRedBCST: nil)
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::ICMS900.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.orig).to eql         2
        expect(subject.csosn).to eql        "900"
        expect(subject.modBC).to eql        3
        expect(subject.vBC).to eql          12345.67
        expect(subject.pRedBC).to be_nil
        expect(subject.pICMS).to eql        234.56
        expect(subject.vICMS).to eql        23456.78
        expect(subject.modBCST).to eql      5
        expect(subject.pMVAST).to be_nil
        expect(subject.pRedBCST).to be_nil
        expect(subject.vBCST).to eql        91234.56
        expect(subject.pCredSN).to eql      234.56
        expect(subject.vCredICMSSN).to eql  45678.90
      end

      it { is_expected.to be_valid }
    end
  end
end
