require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"
require_relative "../shared_examples_for_tdec_0302"

describe Nfe::Models::Taxes::ICMSPart do
  subject { build(:icmsPart) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :orig }
  it { is_expected.to validate_numericality_of(:orig) }
  it { is_expected.to validate_inclusion_of(:orig).in_array([0, 1, 2]) }

  it { is_expected.to validate_presence_of(:cst) }

  it { is_expected.to validate_inclusion_of(:cst).in_array(["10", "90"]) }

  it { is_expected.to validate_presence_of :modBC }
  it { is_expected.to validate_numericality_of(:modBC) }
  it { is_expected.to validate_inclusion_of(:modBC).in_array([0, 1, 2, 3]) }

  it_should_behave_like "a numeric TDec_1302 field", :vBC
  it_should_behave_like "a numeric TDec_0302Opc field", :pRedBC
  it_should_behave_like "a numeric TDec_0302 field", :pICMS
  it_should_behave_like "a numeric TDec_1302 field", :vICMS
  it_should_behave_like "a numeric TDec_0302Opc field", :pMVAST
  it_should_behave_like "a numeric TDec_0302Opc field", :pRedBCST
  it_should_behave_like "a numeric TDec_1302 field", :vBCST
  it_should_behave_like "a numeric TDec_0302 field", :pICMSST
  it_should_behave_like "a numeric TDec_1302 field", :vICMSST
  it_should_behave_like "a numeric TDec_0302 field", :pBCOp

  it { is_expected.to validate_presence_of :modBCST }
  it { is_expected.to validate_numericality_of(:modBCST) }
  it { is_expected.to validate_inclusion_of(:modBCST).in_array([0, 1, 2, 3, 4, 5]) }

  it { is_expected.to validate_presence_of :ufST }
  it { is_expected.to ensure_length_of(:ufST).is_equal_to(2) }

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<orig>2</orig>"
      xml += "<CST>10</CST>"
      xml += "<modBC>3</modBC>"
      xml += "<vBC>12345.67</vBC>"
      xml += "<pRedBC>123.45</pRedBC>"
      xml += "<pICMS>234.56</pICMS>"
      xml += "<vICMS>23456.78</vICMS>"
      xml += "<modBCST>5</modBCST>"
      xml += "<pMVAST>345.67</pMVAST>"
      xml += "<pRedBCST>456.78</pRedBCST>"
      xml += "<vBCST>91234.56</vBCST>"
      xml += "<pICMSST>678.91</pICMSST>"
      xml += "<vICMSST>12345.67</vICMSST>"
      xml += "<pBCOp>789.01</pBCOp>"
      xml += "<UFST>SP</UFST>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::ICMSPart.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.orig).to eql 2
        expect(subject.cst).to eql "10"
        expect(subject.modBC).to eql 3
        expect(subject.vBC).to eql 12345.67
        expect(subject.pRedBC).to eql 123.45
        expect(subject.pICMS).to eql 234.56
        expect(subject.vICMS).to eql 23456.78
        expect(subject.modBCST).to eql 5
        expect(subject.pMVAST).to eql 345.67
        expect(subject.pRedBCST).to eql 456.78
        expect(subject.vBCST).to eql 91234.56
        expect(subject.pICMSST).to eql 678.91
        expect(subject.vICMSST).to eql 12345.67
        expect(subject.pBCOp).to eql 789.01
        expect(subject.ufST).to eql "SP"
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<orig>2</orig>"
      xml += "<CST>10</CST>"
      xml += "<modBC>3</modBC>"
      xml += "<vBC>12345.67</vBC>"
      xml += "<pICMS>234.56</pICMS>"
      xml += "<vICMS>23456.78</vICMS>"
      xml += "<modBCST>5</modBCST>"
      xml += "<vBCST>91234.56</vBCST>"
      xml += "<pICMSST>678.91</pICMSST>"
      xml += "<vICMSST>12345.67</vICMSST>"
      xml += "<pBCOp>789.01</pBCOp>"
      xml += "<UFST>SP</UFST>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:icmsPart, pRedBC: nil, pMVAST: nil, pRedBCST: nil)
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::ICMSPart.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.orig).to eql 2
        expect(subject.cst).to eql "10"
        expect(subject.modBC).to eql 3
        expect(subject.vBC).to eql 12345.67
        expect(subject.pRedBC).to be_nil
        expect(subject.pICMS).to eql 234.56
        expect(subject.vICMS).to eql 23456.78
        expect(subject.modBCST).to eql 5
        expect(subject.pMVAST).to be_nil
        expect(subject.pRedBCST).to be_nil
        expect(subject.vBCST).to eql 91234.56
        expect(subject.pICMSST).to eql 678.91
        expect(subject.vICMSST).to eql 12345.67
        expect(subject.pBCOp).to eql 789.01
        expect(subject.ufST).to eql "SP"
      end

      it { is_expected.to be_valid }
    end
  end
end
