require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Taxes::ICMSST do
  subject { build(:icmsST) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :orig }
  it { is_expected.to validate_numericality_of(:orig) }
  it { is_expected.to validate_inclusion_of(:orig).in_array([0, 1, 2]) }

  it { is_expected.to respond_to "cst" }

  describe "#CST" do
    it "returns always 41" do
      expect(subject.cst).to eql "41"
    end
  end

  it_should_behave_like "a numeric TDec_1302 field", :vBCSTRet
  it_should_behave_like "a numeric TDec_1302 field", :vICMSSTRet
  it_should_behave_like "a numeric TDec_1302 field", :vBCSTDest
  it_should_behave_like "a numeric TDec_1302 field", :vICMSSTDest

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<orig>2</orig>"
      xml += "<CST>41</CST>"
      xml += "<vBCSTRet>12345.67</vBCSTRet>"
      xml += "<vICMSSTRet>23456.78</vICMSSTRet>"
      xml += "<vBCSTDest>34567.89</vBCSTDest>"
      xml += "<vICMSSTDest>45678.90</vICMSSTDest>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::ICMSST.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.orig).to eql 2
        expect(subject.cst).to eql "41"
        expect(subject.vBCSTRet).to eql 12345.67
        expect(subject.vICMSSTRet).to eql 23456.78
        expect(subject.vBCSTDest).to eql 34567.89
        expect(subject.vICMSSTDest).to eql 45678.90
      end

      it { is_expected.to be_valid }
    end
  end
end
