require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Taxes::ICMSTot do
  subject { build :icmsTot }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it_should_behave_like "a numeric TDec_1302 field", :vBC
  it_should_behave_like "a numeric TDec_1302 field", :vICMS
  it_should_behave_like "a numeric TDec_1302 field", :vICMSDeson
  it_should_behave_like "a numeric TDec_1302 field", :vBCST
  it_should_behave_like "a numeric TDec_1302 field", :vST
  it_should_behave_like "a numeric TDec_1302 field", :vProd
  it_should_behave_like "a numeric TDec_1302 field", :vFrete
  it_should_behave_like "a numeric TDec_1302 field", :vSeg
  it_should_behave_like "a numeric TDec_1302 field", :vDesc
  it_should_behave_like "a numeric TDec_1302 field", :vII
  it_should_behave_like "a numeric TDec_1302 field", :vIPI
  it_should_behave_like "a numeric TDec_1302 field", :vPIS
  it_should_behave_like "a numeric TDec_1302 field", :vCOFINS
  it_should_behave_like "a numeric TDec_1302 field", :vOutro
  it_should_behave_like "a numeric TDec_1302 field", :vNF
  it_should_behave_like "a numeric TDec_1302Opc field", :vTotTrib

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<vBC>12345.67</vBC>"
      xml += "<vICMS>23456.78</vICMS>"
      xml += "<vICMSDeson>23456.78</vICMSDeson>"
      xml += "<vBCST>34567.89</vBCST>"
      xml += "<vST>45678.90</vST>"
      xml += "<vProd>12345.67</vProd>"
      xml += "<vFrete>23456.78</vFrete>"
      xml += "<vSeg>34567.89</vSeg>"
      xml += "<vDesc>45678.90</vDesc>"
      xml += "<vII>91234.56</vII>"
      xml += "<vIPI>78901.23</vIPI>"
      xml += "<vPIS>45678.90</vPIS>"
      xml += "<vCOFINS>89123.45</vCOFINS>"
      xml += "<vOutro>67890.12</vOutro>"
      xml += "<vNF>34567.89</vNF>"
      xml += "<vTotTrib>100.45</vTotTrib>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { described_class.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.vBC).to eql 12345.67
        expect(subject.vICMS).to eql 23456.78
        expect(subject.vICMSDeson).to eql 23456.78
        expect(subject.vBCST).to eql 34567.89
        expect(subject.vST).to eql 45678.90
        expect(subject.vProd).to eql 12345.67
        expect(subject.vFrete).to eql 23456.78
        expect(subject.vSeg).to eql 34567.89
        expect(subject.vDesc).to eql 45678.90
        expect(subject.vII).to eql 91234.56
        expect(subject.vIPI).to eql 78901.23
        expect(subject.vPIS).to eql 45678.90
        expect(subject.vCOFINS).to eql 89123.45
        expect(subject.vOutro).to eql 67890.12
        expect(subject.vNF).to eql 34567.89
        expect(subject.vTotTrib).to eql 100.45
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<vBC>12345.67</vBC>"
      xml += "<vICMS>23456.78</vICMS>"
      xml += "<vICMSDeson>23456.78</vICMSDeson>"
      xml += "<vBCST>34567.89</vBCST>"
      xml += "<vST>45678.90</vST>"
      xml += "<vProd>12345.67</vProd>"
      xml += "<vFrete>23456.78</vFrete>"
      xml += "<vSeg>34567.89</vSeg>"
      xml += "<vDesc>45678.90</vDesc>"
      xml += "<vII>91234.56</vII>"
      xml += "<vIPI>78901.23</vIPI>"
      xml += "<vPIS>45678.90</vPIS>"
      xml += "<vCOFINS>89123.45</vCOFINS>"
      xml += "<vOutro>67890.12</vOutro>"
      xml += "<vNF>34567.89</vNF>"
    end

    describe "#to_xml" do
      subject { build :icmsTot, vTotTrib: nil }

      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { described_class.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.vTotTrib).to be_blank
      end

      it { is_expected.to be_valid }
    end
  end
end
