require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Taxes::II do
  subject { build(:ii) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it_should_behave_like "a numeric TDec_1302 field", :vBC
  it_should_behave_like "a numeric TDec_1302 field", :vDespAdu
  it_should_behave_like "a numeric TDec_1302 field", :vII
  it_should_behave_like "a numeric TDec_1302 field", :vIOF

  describe "#present?" do
    subject { Nfe::Models::Taxes::II.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.vBC = 1 }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<vBC>12345.67</vBC>"
      xml += "<vDespAdu>23456.78</vDespAdu>"
      xml += "<vII>34567.89</vII>"
      xml += "<vIOF>45678.90</vIOF>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::II.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.vBC).to eql 12345.67
        expect(subject.vDespAdu).to eql 23456.78
        expect(subject.vII).to eql 34567.89
        expect(subject.vIOF).to eql 45678.90
      end

      it { is_expected.to be_valid }
    end
  end
end
