require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"
require_relative "../shared_examples_for_tdec_1204"
require_relative "../shared_examples_for_tdec_1104"
require_relative "../shared_examples_for_tdec_0302"

describe Nfe::Models::Taxes::IPITrib do
  subject { build(:ipiTrib) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of(:cst) }
  it { is_expected.to validate_inclusion_of(:cst).in_array(["00", "49", "50", "99"]) }

  it_should_behave_like "a numeric TDec_1302 field", :vBC
  it_should_behave_like "a numeric TDec_0302 field", :pIPI
  it_should_behave_like "a numeric TDec_1204 field", :qUnid
  it_should_behave_like "a numeric TDec_1104v field", :vUnid
  it_should_behave_like "a numeric TDec_1302 field", :vIPI

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<CST>50</CST>"
      xml += "<vBC>12345.67</vBC>"
      xml += "<pIPI>988.76</pIPI>"
      xml += "<qUnid>123.6600</qUnid>"
      xml += "<vUnid>78.6500</vUnid>"
      xml += "<vIPI>23456.78</vIPI>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::IPITrib.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cst).to eql "50"
        expect(subject.vBC).to eql 12345.67
        expect(subject.pIPI).to eql 988.76
        expect(subject.qUnid).to eql 123.6600
        expect(subject.vUnid).to eql 78.6500
        expect(subject.vIPI).to eql 23456.78
      end

      it { is_expected.to be_valid }
    end
  end
end
