require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"
require_relative "../shared_examples_for_tdec_0302"

describe Nfe::Models::Taxes::ISSQN do
  subject { build(:issqn) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it_should_behave_like "a numeric TDec_1302 field", :vBC
  it_should_behave_like "a numeric TDec_0302 field", :vAliq
  it_should_behave_like "a numeric TDec_1302 field", :vISSQN

  describe "#present?" do
    subject { Nfe::Models::Taxes::ISSQN.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.vBC = 1 }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<vBC>12345.67</vBC>"
      xml += "<vAliq>234.56</vAliq>"
      xml += "<vISSQN>34567.89</vISSQN>"
      xml += "<cMunFG>1234567</cMunFG>"
      xml += "<cListServ>101</cListServ>"
      xml += "<cSitTrib>N</cSitTrib>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::ISSQN.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.vBC).to eql 12345.67
        expect(subject.vAliq).to eql 234.56
        expect(subject.vISSQN).to eql 34567.89
        expect(subject.cMunFG).to eql "1234567"
        expect(subject.cListServ).to eql "101"
        expect(subject.cSitTrib).to eql "normal"
      end

      it { is_expected.to be_valid }
    end
  end
end
