require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Taxes::ISSQNtot do
  subject { build(:issqntot) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it_should_behave_like "a numeric TDec_1302Opc field", :vServ
  it_should_behave_like "a numeric TDec_1302Opc field", :vBC
  it_should_behave_like "a numeric TDec_1302Opc field", :vISS
  it_should_behave_like "a numeric TDec_1302Opc field", :vPIS
  it_should_behave_like "a numeric TDec_1302Opc field", :vCOFINS

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<vServ>12345.67</vServ>"
      xml += "<vBC>23456.78</vBC>"
      xml += "<vISS>34567.89</vISS>"
      xml += "<vPIS>91234.56</vPIS>"
      xml += "<vCOFINS>78912.34</vCOFINS>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::ISSQNtot.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.vServ).to eql 12345.67
        expect(subject.vBC).to eql 23456.78
        expect(subject.vISS).to eql 34567.89
        expect(subject.vPIS).to eql 91234.56
        expect(subject.vCOFINS).to eql 78912.34
      end

      it { is_expected.to be_valid }
    end
  end

 context "when xml doesn't have optional tags" do
    subject { Nfe::Models::Taxes::ISSQNtot.new }

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to_not be_present
      end
    end

    describe "#parse!" do
      before { subject.parse!("<root></root>") }

      it "parses xml" do
        expect(subject.vServ).to be_nil
        expect(subject.vBC).to be_nil
        expect(subject.vISS).to be_nil
        expect(subject.vPIS).to be_nil
        expect(subject.vCOFINS).to be_nil
      end

      it { is_expected.to be_valid }
    end
  end

end
