require "spec_helper"
require_relative "../shared_examples_for_tdec_0302"
require_relative "../shared_examples_for_tdec_1104"
require_relative "../shared_examples_for_tdec_1204"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Taxes::PISCOFINSST do
  subject { build(:cofinsst) }

  it_should_behave_like "a numeric TDec_1302Opc field", :vBC
  it_should_behave_like "a numeric TDec_0302Opc field", :tax
  it_should_behave_like "a numeric TDec_1204Opc field", :qBCProd
  it_should_behave_like "a numeric TDec_1104Opc field", :vAliqProd
  it_should_behave_like "a numeric TDec_1302Opc field", :value

  describe "#present?" do
    subject { described_class.new(nil, nil) }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.vBC = 1 }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<vBC>12345.67</vBC>"
      xml += "<pCOFINS>234.56</pCOFINS>"
      xml += "<qBCProd>34567.8901</qBCProd>"
      xml += "<vAliqProd>45678.9012</vAliqProd>"
      xml += "<vCOFINS>12345.67</vCOFINS>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { described_class.new("pCOFINS", "vCOFINS") }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.vBC).to eql 12345.67
        expect(subject.tax).to eql 234.56
        expect(subject.qBCProd).to eql 34567.8901
        expect(subject.vAliqProd).to eql 45678.9012
        expect(subject.value).to eql 12345.67
      end

      it { is_expected.to be_valid }
    end
  end
end
