require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Taxes::Imposto do
  subject { build(:imposto) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it_should_behave_like "a numeric TDec_1302Opc field", :vTotTrib

  it { is_expected.to validate_presence_of :icms }

  context "when icms is present and invalid" do
    it "returns an error on icms" do
      subject.icms = build(:icms00, orig: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/icms/)
    end
  end

  context "when pis is present and invalid" do
    it "returns an error on pis" do
      subject.pis = build(:pis_when_aliq, cst: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/pis/)
    end
  end

  context "when cofins is present and invalid" do
    it "returns an error on cofins" do
      subject.cofins = build(:cofins_when_aliq, cst: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/cofins/)
    end
  end

  context "when ipi is present and invalid" do
    it "returns an error on ipi" do
      subject.ipi = build(:ipi_when_Trib, cst: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/ipi/)
    end
  end

  context "when ii is present and invalid" do
    it "returns an error on ii" do
      subject.ii = build(:ii, vBC: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/ii/)
    end
  end

  context "when issqn is present and invalid" do
    it "returns an error on issqn" do
      subject.issqn = build(:issqn, vBC: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/issqn/)
    end
  end

  context "when issqn pisst present and invalid" do
    it "returns an error on issqn" do
      subject.pisst = build(:pisst, vBC: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/pisst/)
    end
  end

  context "when issqn cofinsst present and invalid" do
    it "returns an error on cofinsst" do
      subject.cofinsst = build(:cofinsst, vBC: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/cofinsst/)
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<vTotTrib>10.40</vTotTrib>"
      xml += "<ICMS>#{build(:icms_when_ICMS00).to_xml}</ICMS>"
      xml += "<IPI>#{build(:ipi_when_Trib).to_xml}</IPI>"
      xml += "<II>#{build(:ii).to_xml}</II>"
      xml += "<ISSQN>#{build(:issqn).to_xml}</ISSQN>"
      xml += "<PIS>#{build(:pis_when_aliq).to_xml}</PIS>"
      xml += "<PISST>#{build(:pisst).to_xml}</PISST>"
      xml += "<COFINS>#{build(:cofins_when_aliq).to_xml}</COFINS>"
      xml += "<COFINSST>#{build(:cofinsst).to_xml}</COFINSST>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::Imposto.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.vTotTrib).to eql 10.4
        expect(subject.icms.to_xml).to eql build(:icms_when_ICMS00).to_xml
        expect(subject.ipi.to_xml).to eql build(:ipi_when_Trib).to_xml
        expect(subject.ii.to_xml).to eql build(:ii).to_xml
        expect(subject.pis.to_xml).to eql build(:pis_when_aliq).to_xml
        expect(subject.pisst.to_xml).to eql build(:pisst).to_xml
        expect(subject.cofins.to_xml).to eql build(:cofins_when_aliq).to_xml
        expect(subject.cofinsst.to_xml).to eql build(:cofinsst).to_xml
        expect(subject.issqn.to_xml).to eql build(:issqn).to_xml
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    subject { Nfe::Models::Taxes::Imposto.new }

    let(:xml) do
      xml = "<ICMS>#{build(:icms_when_ICMS00).to_xml}</ICMS>"
      xml += "<PIS>#{build(:pis_when_aliq).to_xml}</PIS>"
      xml += "<COFINS>#{build(:cofins_when_aliq).to_xml}</COFINS>"
    end

    describe "#to_xml" do
      before do
        subject.icms = build(:icms_when_ICMS00)
        subject.pis = build(:pis_when_aliq)
        subject.cofins = build(:cofins_when_aliq)
      end

      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.icms.to_xml).to eql build(:icms_when_ICMS00).to_xml
        expect(subject.pis.to_xml).to eql build(:pis_when_aliq).to_xml
        expect(subject.cofins.to_xml).to eql build(:cofins_when_aliq).to_xml
      end

      it { is_expected.to be_valid }
    end
  end
end
