require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Taxes::RetTrib do
  subject { build(:retTrib) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it_should_behave_like "a numeric TDec_1302Opc field", :vRetPIS
  it_should_behave_like "a numeric TDec_1302Opc field", :vRetCOFINS
  it_should_behave_like "a numeric TDec_1302Opc field", :vRetCSLL
  it_should_behave_like "a numeric TDec_1302Opc field", :vBCIRRF
  it_should_behave_like "a numeric TDec_1302Opc field", :vIRRF
  it_should_behave_like "a numeric TDec_1302Opc field", :vBCRetPrev

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<vRetPIS>12345.67</vRetPIS>"
      xml += "<vRetCOFINS>23456.78</vRetCOFINS>"
      xml += "<vRetCSLL>34567.89</vRetCSLL>"
      xml += "<vBCIRRF>91234.56</vBCIRRF>"
      xml += "<vIRRF>78912.34</vIRRF>"
      xml += "<vBCRetPrev>56789.01</vBCRetPrev>"
      xml += "<vRetPrev>67890.12</vRetPrev>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Taxes::RetTrib.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.vRetPIS).to eql 12345.67
        expect(subject.vRetCOFINS).to eql 23456.78
        expect(subject.vRetCSLL).to eql 34567.89
        expect(subject.vBCIRRF).to eql 91234.56
        expect(subject.vIRRF).to eql 78912.34
        expect(subject.vBCRetPrev).to eql 56789.01
        expect(subject.vRetPrev).to eql 67890.12
      end

      it { is_expected.to be_valid }
    end
  end

 context "when xml doesn't have optional tags" do
    subject { Nfe::Models::Taxes::RetTrib.new }

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to_not be_present
      end
    end

    describe "#parse!" do
      before { subject.parse!("<root></root>") }

      it "parses xml" do
        expect(subject.vRetPIS).to be_nil
        expect(subject.vRetCOFINS).to be_nil
        expect(subject.vRetCSLL).to be_nil
        expect(subject.vBCIRRF).to be_nil
        expect(subject.vIRRF).to be_nil
        expect(subject.vBCRetPrev).to be_nil
        expect(subject.vRetPrev).to be_nil
      end

      it { is_expected.to be_valid }
    end
  end
end
