shared_examples "a COFINSOutr" do |factory, root|
  subject { build(factory) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = ""
      xml += "<#{root}>" if root.present?
      xml += "<CST>99</CST>"
      xml += "<vBC>98765.43</vBC>"
      xml += "<pCOFINS>123.45</pCOFINS>"
      xml += "<qBCProd>12345.6789</qBCProd>"
      xml += "<vAliqProd>78901.2345</vAliqProd>"
      xml += "<vCOFINS>23456.78</vCOFINS>"
      xml += "</#{root}>" if root.present?

      xml
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { new_object }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cst).to eql "99"
        expect(subject.vBC).to eql 98765.43
        expect(subject.tax).to eql 123.45
        expect(subject.qBCProd).to eql 12345.6789
        expect(subject.vAliqProd).to eql 78901.2345
        expect(subject.value).to eql 23456.78
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml has tax tags" do
    let(:xml) do
      xml  = ""
      xml += "<#{root}>" if root.present?
      xml += "<CST>99</CST>"
      xml += "<vBC>98765.43</vBC>"
      xml += "<pCOFINS>123.45</pCOFINS>"
      xml += "<vCOFINS>23456.78</vCOFINS>"
      xml += "</#{root}>" if root.present?

      xml
    end

    describe "#to_xml" do
      subject { build(factory, qBCProd: nil, vAliqProd: nil) }
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { new_object }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cst).to eql "99"
        expect(subject.vBC).to eql 98_765.43
        expect(subject.tax).to eql 123.45
        expect(subject.value).to eql 23_456.78
        expect(subject.qBCProd).to be_nil
        expect(subject.vAliqProd).to be_nil
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml has value tags" do
    let(:xml) do
      xml  = ""
      xml += "<#{root}>" if root.present?
      xml += "<CST>99</CST>"
      xml += "<qBCProd>12345.6789</qBCProd>"
      xml += "<vAliqProd>78901.2345</vAliqProd>"
      xml += "<vCOFINS>23456.78</vCOFINS>"
      xml += "</#{root}>" if root.present?

      xml
    end

    describe "#to_xml" do
      subject { build(factory, tax: nil, vBC: nil) }
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { new_object }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cst).to eql "99"
        expect(subject.qBCProd).to eql 12345.6789
        expect(subject.vAliqProd).to eql 78901.2345
        expect(subject.value).to eql 23_456.78
        expect(subject.vBC).to be_nil
        expect(subject.tax).to be_nil
      end

      it { is_expected.to be_valid }
    end
  end
end
