require "spec_helper"
require_relative "../shared_examples_for_tdec_0302"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Transport::RetTransp do
  subject { build(:retTransp) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it_should_behave_like "a numeric TDec_1302 field", :vServ
  it_should_behave_like "a numeric TDec_1302 field", :vBCRet
  it_should_behave_like "a numeric TDec_0302 field", :pICMSRet
  it_should_behave_like "a numeric TDec_1302 field", :vICMSRet

  it { is_expected.to ensure_length_of(:cfop).is_equal_to 4 }
  it { is_expected.to validate_numericality_of(:cfop).only_integer }

  it { is_expected.to ensure_length_of(:cMunFG).is_equal_to 7 }
  it { is_expected.to validate_numericality_of(:cMunFG).only_integer }

  describe "#present?" do
    subject { Nfe::Models::Transport::RetTransp.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.vServ = 1 }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<vServ>98765.43</vServ>"
      xml += "<vBCRet>12345.67</vBCRet>"
      xml += "<pICMSRet>345.67</pICMSRet>"
      xml += "<vICMSRet>1234567.89</vICMSRet>"
      xml += "<CFOP>5351</CFOP>"
      xml += "<cMunFG>4218004</cMunFG>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Transport::RetTransp.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.vServ).to eql 98765.43
        expect(subject.vBCRet).to eql 12345.67
        expect(subject.pICMSRet).to eql 345.67
        expect(subject.vICMSRet).to eql 1234567.89
        expect(subject.cfop).to eql "5351"
        expect(subject.cMunFG).to eql "4218004"
      end

      it { is_expected.to be_valid }
    end
  end
end
