require "spec_helper"

describe Nfe::Models::Transport::Transporta do
  subject { build(:transporta) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  describe "#CNPJ" do
    it { is_expected.to allow_value("").for(:cnpj) }
    it { is_expected.to ensure_length_of(:cnpj).is_equal_to(14).with_message(/não possui o tamanho esperado/) }

    context "when number is invalid" do
      before { subject.cnpj = "0" }
      it { is_expected.not_to be_valid }
    end
  end

  describe "#CPF" do
    it { is_expected.to allow_value("").for(:cpf) }
    it { is_expected.to ensure_length_of(:cpf).is_equal_to(11).with_message(/não possui o tamanho esperado/) }

    context "when number is invalid" do
      before { subject.cpf = "0" }
      it { is_expected.not_to be_valid }
    end
  end

  it { is_expected.to allow_value("").for(:xNome) }
  it { is_expected.to ensure_length_of(:xNome).is_at_most(60) }

  describe "#IE" do
    it { is_expected.to allow_value("").for(:ie) }
    it { is_expected.to ensure_length_of(:ie).is_at_least(2).is_at_most(14) }

    context "when number is invalid" do
      before { subject.ie = "1234" }
      it { is_expected.not_to be_valid }
    end
  end

  it { is_expected.to allow_value("").for(:xEnder) }
  it { is_expected.to ensure_length_of(:xEnder).is_at_most(60) }

  it { is_expected.to allow_value("").for(:xMun) }
  it { is_expected.to ensure_length_of(:xMun).is_at_most(60) }

  it { is_expected.to allow_value("").for(:uf) }
  it { is_expected.to ensure_length_of(:uf).is_equal_to(2) }

  describe "#present?" do
    subject { Nfe::Models::Transport::Transporta.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.uf = "SC" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<CNPJ>01144012000152</CNPJ>"
      xml += "<CPF>11144477735</CPF>"
      xml += "<xNome>Razao Social</xNome>"
      xml += "<IE>253263131</IE>"
      xml += "<xEnder>Endereco</xEnder>"
      xml += "<xMun>Municipio</xMun>"
      xml += "<UF>SC</UF>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Transport::Transporta.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cnpj ).to eql "01144012000152"
        expect(subject.cpf).to eql "11144477735"
        expect(subject.xNome).to eql "Razao Social"
        expect(subject.ie).to eql "253263131"
        expect(subject.xEnder).to eql "Endereco"
        expect(subject.xMun).to eql "Municipio"
        expect(subject.uf).to eql "SC"
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    subject { Nfe::Models::Transport::Transporta.new }

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to_not be_present
      end

      it { is_expected.to be_valid }
    end

    describe "#parse!" do
      before { subject.parse!("<root></root>") }

      it "parses xml values to itself" do
        expect(subject.cnpj ).to be_nil
        expect(subject.cpf).to be_nil
        expect(subject.xNome).to be_nil
        expect(subject.ie).to be_nil
        expect(subject.xEnder).to be_nil
        expect(subject.xMun).to be_nil
        expect(subject.uf).to be_nil
      end

      it { is_expected.to be_valid }
    end
  end
end
