require "spec_helper"

describe Nfe::Models::Transport::VeicTranspReboque do
  subject { build(:veicTranspReboque) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:placa).is_equal_to(7) }

  it { is_expected.to ensure_length_of(:uf).is_equal_to(2) }

  it { is_expected.to allow_value("", nil).for(:rntc) }
  it { is_expected.to ensure_length_of(:rntc).is_at_most(20) }

  describe "#present?" do
    subject { described_class.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.uf = "SC" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<placa>XXP9687</placa>"
      xml += "<UF>RJ</UF>"
      xml += "<RNTC>3265981358AA</RNTC>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { described_class.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.placa).to eql "XXP9687"
        expect(subject.uf).to eql "RJ"
        expect(subject.rntc).to eql "3265981358AA"
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<placa>XXP9687</placa>"
      xml += "<UF>RJ</UF>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:veicTranspReboque, rntc: nil)
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject {  described_class.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.placa).to eql "XXP9687"
        expect(subject.uf).to eql "RJ"
      end

      it { is_expected.to be_valid }
    end
  end
end
