require "spec_helper"
require_relative "../shared_examples_for_tdec_1302"

describe Nfe::Models::Transport::Vol do
  subject { build(:vol) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to allow_value("", nil).for(:qVol) }
  it { is_expected.to validate_numericality_of(:qVol) }
  it { is_expected.to ensure_length_of(:qVol).is_at_most(15) }

  it { is_expected.to allow_value("", nil).for(:esp) }
  it { is_expected.to ensure_length_of(:esp).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:marca) }
  it { is_expected.to ensure_length_of(:marca).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:nVol) }
  it { is_expected.to ensure_length_of(:nVol).is_at_most(60) }

  it_should_behave_like "a numeric TDec_1302 field", :pesoL
  it_should_behave_like "a numeric TDec_1302 field", :pesoB

  context "when lacres is present and invalid" do
    it "returns an error on lacres" do
      subject.lacres << build(:lacres, nLacre: "A"*61)
      subject.valid?

      expect(subject.errors.to_json).to match(/lacres/)
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<qVol>10</qVol>"
      xml += "<esp>Piso</esp>"
      xml += "<marca>Portobello</marca>"
      xml += "<nVol>0123474</nVol>"
      xml += "<pesoL>123456.780</pesoL>"
      xml += "<pesoB>234567.890</pesoB>"
      xml += "<lacres><nLacre>Texto0</nLacre></lacres>"
      xml += "<lacres><nLacre>Texto1</nLacre></lacres>"
      xml
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Transport::Vol.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.qVol).to eql 10
        expect(subject.esp).to eql "Piso"
        expect(subject.marca).to eql "Portobello"
        expect(subject.nVol).to eql "0123474"
        expect(subject.pesoL).to eql 123456.780
        expect(subject.pesoB).to eql 234567.890
        expect(subject.lacres[0].nLacre).to eql "Texto0"
        expect(subject.lacres[1].nLacre).to eql "Texto1"
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    subject { Nfe::Models::Transport::Vol.new }

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to_not be_present
      end
    end

    describe "#parse!" do
      before { subject.parse!("<root></root>") }

      it "parses xml" do
        expect(subject.qVol).to be_nil
        expect(subject.esp).to be_blank
        expect(subject.marca).to be_blank
        expect(subject.nVol).to be_blank
        expect(subject.pesoL).to be_nil
        expect(subject.pesoB).to be_nil
        expect(subject.lacres).to be_blank
      end
    end
  end
end
