require "spec_helper"

shared_examples "an operation" do | fixture, object_send, object_result |
  let(:webservice) { Nfe::Webservices::Sender }

  let(:xml) do
    Nokogiri::XML(File.read("spec/nfe/webservices/fixtures/#{fixture}.xml"))
  end

  let(:errors) do
    errors = ActiveModel::Errors.new(described_class)
    errors.add(:base, "x")
    errors
  end

  let(:invalid) { Nokogiri::XML::Document.new }

  describe "#execute" do
    context "when works" do
      before do
        allow_any_instance_of(webservice).to receive(:execute).and_return(true)
        allow_any_instance_of(webservice).to receive(:result).and_return(xml)
      end

      it "returns true" do
        expect(operation.execute).to be_truthy
      end

      it "fills a #{object_result} object" do
        operation.execute
        expect(operation.result).to be_an_instance_of object_result
      end
    end

    context "when webservice don't work" do
      before do
        allow_any_instance_of(webservice).to receive(:execute).and_return(false)
        allow_any_instance_of(webservice).to receive(:errors).and_return(errors)
      end

      it "returns false" do
        expect(operation.execute).to be_falsey
      end

      it "fills an errors object" do
        operation.execute
        expect(operation.errors).to_not be_empty
      end
    end

    context "when result xml is unexpected" do
      before do
        allow_any_instance_of(webservice).to receive(:execute).and_return(true)
        allow_any_instance_of(webservice).to receive(:result).and_return(invalid)
      end

      it "returns false" do
        expect(operation.execute).to be_falsey
      end

      it "fills an errors object" do
        operation.execute
        expect(operation.errors).to_not be_empty
      end
    end

    context "when object has invalid attributes" do
      before do
        allow_any_instance_of(webservice).to receive(:execute).and_return(true)
        allow_any_instance_of(webservice).to receive(:result).and_return(xml)
        allow_any_instance_of(object_send).to receive(:versao).and_return("XX")
      end

      it "returns false" do
        expect(operation.execute).to be_falsey
      end

      it "fills an errors object" do
        operation.execute
        expect(operation.errors).to_not be_empty
      end
    end
  end
end
