require "spec_helper"

describe Nfe::Services::Dispatch::ConsCad do
  subject { build(:consCad) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:versao).is_equal_to(4) }

  it { is_expected.to validate_presence_of :uf }

  it { is_expected.to ensure_length_of(:uf).is_equal_to(2) }

  describe "#IE" do
    context "when CNPJ is present" do
      before { subject.cnpj = "01144012000152" }
      it { is_expected.to allow_value("").for(:ie) }
    end

    context "when number is invalid" do
      before { subject.ie = "1234" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:ie).is_at_least(2).is_at_most(14) }
  end

  describe "#CPF" do
    context "when CNPJ is present" do
      before { subject.cnpj = "01144012000152" }
      it { is_expected.to allow_value("").for(:cpf) }
    end

    context "when number is invalid" do
      before { subject.cpf = "11111111122" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:cpf).is_equal_to(11) }
  end

  describe "#CNPJ" do
    context "when cpf is present" do
      before { subject.cpf = "11144477735" }
      it { is_expected.to allow_value("").for(:cnpj) }
    end

    context "when number is invalid" do
      before { subject.cnpj = "04001155000103" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:cnpj).is_equal_to(14) }
  end

  it { is_expected.to validate_presence_of :versao }
  it { is_expected.to ensure_length_of(:versao).is_equal_to(4) }

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<ConsCad xmlns=\"http://www.portalfiscal.inf.br/nfe\" versao=\"2.00\">"
      xml += "<infCons>"
      xml += "<xServ>CONS-CAD</xServ>"
      xml += "<UF>SC</UF>"
      xml += "<IE>253263131</IE>"
      xml += "<CNPJ>01144012000152</CNPJ>"
      xml += "<CPF>11144477735</CPF>"
      xml += "</infCons>"
      xml += "</ConsCad>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end
  end
end
