require "spec_helper"

describe Nfe::Services::Dispatch::InutNFe do
  subject { build(:inutNFe) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:versao).is_equal_to(4) }

  it { is_expected.to ensure_length_of(:cUF).is_equal_to(2) }

  it { is_expected.to ensure_length_of(:ano).is_equal_to(2) }

  it { is_expected.to ensure_length_of(:cnpj).is_equal_to(14) }

  it { is_expected.to ensure_length_of(:serie).is_at_least(1).is_at_most(3) }

  it { is_expected.to ensure_length_of(:nNFIni).is_at_least(1).is_at_most(9) }

  it { is_expected.to ensure_length_of(:nNFFin).is_at_least(1).is_at_most(9) }

  it { is_expected.to ensure_length_of(:xJust).is_at_least(15).is_at_most(255) }

  let(:xml) do
    xml  = "<inutNFe xmlns=\"http://www.portalfiscal.inf.br/nfe\" versao=\"2.00\">"
    xml +=   "<infInut Id=\"ID43130114401200015255123000000001000000002\">"
    xml +=     "<tpAmb>1</tpAmb>"
    xml +=     "<xServ>INUTILIZAR</xServ>"
    xml +=     "<cUF>43</cUF>"
    xml +=     "<ano>13</ano>"
    xml +=     "<CNPJ>01144012000152</CNPJ>"
    xml +=     "<mod>55</mod>"
    xml +=     "<serie>123</serie>"
    xml +=     "<nNFIni>1</nNFIni>"
    xml +=     "<nNFFin>2</nNFFin>"
    xml +=     "<xJust>Erro de digitação</xJust>"
    xml +=   "</infInut>"
    xml +=   "<Signature xmlns=\"#{Nfe::Models::Signature.namespace}\">"
    xml +=     "#{build(:signature).value}"
    xml +=   "</Signature>"
    xml += "</inutNFe>"
  end

  describe "#to_xml" do
    it "returns a formatted xml" do
      expect(subject.to_xml).to eql xml
    end
  end
end
