require "spec_helper"

describe Nfe::Services::Regress::ProtNFe do
  let(:id) { " Id=\"ID342140000245084\"" }

  let(:xml) do
    "<protNFe versao=\"2.00\">"\
      "<infProt#{id}>"\
        "<tpAmb>2</tpAmb>"\
        "<verAplic>SVRS20140401105551</verAplic>"\
        "<chNFe>42140301144012000152550010000000021458965750</chNFe>"\
        "<dhRecbto>2014-04-07T11:50:44-03:00</dhRecbto>"\
        "<nProt>342140000245084</nProt>"\
        "<digVal>M7Mu1XMg2i9/0R9IoUncq5MjT+8=</digVal>"\
        "<cStat>100</cStat>"\
        "<xMotivo>Autorizado o uso da NF-e</xMotivo>"\
      "</infProt>"\
    "</protNFe>"
  end

  describe "#parse!" do
    before { subject.parse! xml }

    it "parses xml values to itself" do
      expect(subject.versao).to eql "2.00"
      expect(subject.id).to eql "ID342140000245084"
      expect(subject.tpAmb).to eql "2"
      expect(subject.verAplic).to eql "SVRS20140401105551"
      expect(subject.chNFe).to eql "42140301144012000152550010000000021458965750"
      expect(subject.dhRecbto).to eql DateTime.strptime("2014-04-07T11:50:44-03:00", "%Y-%m-%dT%H:%M:%S%:z")
      expect(subject.nProt).to eql "342140000245084"
      expect(subject.digVal).to eql "M7Mu1XMg2i9/0R9IoUncq5MjT+8="
      expect(subject.cStat).to eql 100
      expect(subject.xMotivo).to eql "Autorizado o uso da NF-e"
    end
  end

  describe "#to_xml" do
    context "when has ID" do
      subject { build(:protNFe) }

      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    context "when doesn't have ID" do
      let (:id) {""}
      subject { build(:protNFe, id: nil) }

      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end
  end
end
