require "spec_helper"

describe Nfe::Services::Regress::RetEvento do
  let(:xml) do
    xml  = "<retEvento versao=\"1.00\">"
    xml +=   "<infEvento Id=\"ID123456\">"
    xml +=     "<tpAmb>2</tpAmb>"
    xml +=     "<verAplic>RS20131127100609</verAplic>"
    xml +=     "<cOrgao>43</cOrgao>"
    xml +=     "<cStat>297</cStat>"
    xml +=     "<xMotivo>Rejeicao: Assinatura difere do calculado</xMotivo>"
    xml +=     "<chNFe>31101203913809000100550010000000221000000220</chNFe>"
    xml +=     "<tpEvento>110111</tpEvento>"
    xml +=     "<xEvento>Carta de Correção registrada</xEvento>"
    xml +=     "<nSeqEvento>1</nSeqEvento>"
    xml +=     "<dhRegEvento>2014-02-06T16:46:26-02:00</dhRegEvento>"
    xml +=     "<nProt>342150000069742</nProt>"
    xml +=   "</infEvento>"
    xml += "</retEvento>"
  end

  describe "#parse!" do
    before { subject.parse! xml }

    it "parses xml values to itself" do
      expect(subject.versao).to eql "1.00"
      expect(subject.id).to eql "ID123456"
      expect(subject.tpAmb).to eql "test"
      expect(subject.verAplic).to eql "RS20131127100609"
      expect(subject.cOrgao).to eql "43"
      expect(subject.cStat).to eql 297
      expect(subject.xMotivo).to eql "Rejeicao: Assinatura difere do calculado"
      expect(subject.chNFe).to eql "31101203913809000100550010000000221000000220"
      expect(subject.tpEvento).to eql "110111"
      expect(subject.nSeqEvento).to eql "1"
      expect(subject.nProt).to eql "342150000069742"
      expect(subject.dhRegEvento).to eql DateTime.
        strptime("2014-02-06T16:46:26-02:00", "%Y-%m-%dT%H:%M:%S%:z")

    end

    it { is_expected.to be_valid }
  end

  describe "#to_xml" do
    before do
      subject.versao = "1.00"
      subject.id = "ID123456"
      subject.tpAmb = "test"
      subject.verAplic = "RS20131127100609"
      subject.cOrgao = "43"
      subject.cStat = 297
      subject.xMotivo = "Rejeicao: Assinatura difere do calculado"
      subject.chNFe = "31101203913809000100550010000000221000000220"
      subject.tpEvento = "110111"
      subject.nSeqEvento = "1"
      subject.xEvento = "Carta de Correção registrada"
      subject.nProt = "342150000069742"
      subject.dhRegEvento = DateTime.strptime("2014-02-06T16:46:26-02:00", "%Y-%m-%dT%H:%M:%S%:z")
    end

    it "returns a formatted xml" do
      expect(subject.to_xml).to eql xml
    end
  end
end
