require "spec_helper"

describe Nfe::Services::Regress::RetInutNFe do

  let(:xml) do
    xml  = "<retInutNFe versao=\"2.00\">"
    xml +=   "<infInut Id=\"ID12345678912345678912345678912345678912345\">"
    xml +=   "<tpAmb>1</tpAmb>"
    xml +=   "<verAplic>SVAN</verAplic>"
    xml +=   "<cStat>123</cStat>"
    xml +=   "<xMotivo>Erro de digitação</xMotivo>"
    xml +=   "<cUF>43</cUF>"
    xml +=   "<ano>13</ano>"
    xml +=   "<CNPJ>01144012000152</CNPJ>"
    xml +=   "<mod>55</mod>"
    xml +=   "<serie>123</serie>"
    xml +=   "<nNFIni>1</nNFIni>"
    xml +=   "<nNFFin>2</nNFFin>"
    xml +=   "<dhRecbto>2013-12-05T12:34:56-03:00</dhRecbto>"
    xml +=   "<nProt>131100023074367</nProt>"
    xml +=   "</infInut>"
    xml += "</retInutNFe>"
  end

  describe "#parse!" do
    before { subject.parse! xml }

    it "parses xml values to itself" do
      expect(subject.versao).to eql "2.00"
      expect(subject.id).to eql "ID12345678912345678912345678912345678912345"
      expect(subject.tpAmb).to eql "production"
      expect(subject.verAplic).to eql "SVAN"
      expect(subject.cStat).to eql 123
      expect(subject.xMotivo).to eql "Erro de digitação"
      expect(subject.cUF).to eql "43"
      expect(subject.ano).to eql "13"
      expect(subject.cnpj ).to eql "01144012000152"
      expect(subject.mod).to eql "55"
      expect(subject.serie).to eql "123"
      expect(subject.nNFIni).to eql "1"
      expect(subject.nNFFin).to eql "2"
      expect(subject.dhRecbto).to eql DateTime.strptime("2013-12-05T12:34:56-03:00", "%Y-%m-%dT%H:%M:%S%:z")
      expect(subject.nProt).to eql "131100023074367"
    end
  end

  describe "#to_xml" do
    before do
      subject.versao= "2.00"
      subject.id= "ID12345678912345678912345678912345678912345"
      subject.tpAmb= "production"
      subject.verAplic= "SVAN"
      subject.cStat= 123
      subject.xMotivo= "Erro de digitação"
      subject.cUF= "43"
      subject.ano= "13"
      subject.cnpj= "01144012000152"
      subject.mod= "55"
      subject.serie= "123"
      subject.nNFIni= "1"
      subject.nNFFin= "2"
      subject.dhRecbto= DateTime.strptime("2013-12-05T12:34:56-03:00", "%Y-%m-%dT%H:%M:%S%:z")
      subject.nProt= "131100023074367"
    end

    it "returns a formatted xml" do
      expect(subject.to_xml).to eql xml
    end
  end
end
