require "spec_helper"

describe Nfe::Webservices::Sender do

  let(:object) { build(:consStatServ) }
  let(:ws_info) { Nfe::Webservices::Wsinfo::ServiceStatus.new(:test, "SC") }

  subject { described_class.new(nil, nil, ws_info) }

  let(:returned_xml) do
    arquivo = File.read("spec/nfe/webservices/fixtures/retConsStatServ.xml")
    Nokogiri::XML(arquivo)
  end

  describe "#execute" do
    context "when works" do
      before do
        allow_any_instance_of(Nfe::Webservices::ClientSoap).
          to receive(:call).with(object.to_xml).and_return(returned_xml)
      end

      it "returns true" do
        expect(subject.execute(object)).to be_truthy
      end

      it "fills a Nokogiri document with the returned xml" do
        subject.execute(object)
        expect(subject.result).to be_an_instance_of Nokogiri::XML::Document
      end
    end

    context "when occorrer a communication error" do
      before do
        allow_any_instance_of(Nfe::Webservices::ClientSoap).
          to receive(:call).with(object.to_xml).and_raise(Savon::Error)
      end

      it "returns false" do
        expect(subject.execute(object)).to be_falsey
      end

      it "fills an errors object" do
        subject.execute(object)
        expect(subject.errors).to_not be_empty
      end
    end

    context "when XML is bigger than 500 Kbytes" do
      before do
        more_than_500_kbytes = "A" * 512001
        allow(object).to receive(:to_xml).and_return(more_than_500_kbytes)
      end

      it "returns false" do
        expect(subject.execute(object)).to be_falsey
      end

      it "fills an errors object" do
        subject.execute(object)
        expect(subject.errors.messages[:base]).to eql [
          t("webservice.dispatch.exceeded_size")]
      end
    end
  end
end
