require "spec_helper"

describe Xmldsig::Canonicalizer do
  shared_examples "a canonicalized xml" do |method|
    let(:doc) do
      Nokogiri.XML <<-eoxml
        <?xml version="1.0"?>
          <!DOCTYPE doc SYSTEM "doc.dtd">
          <doc>
            Hello, world!
            <!-- Comment 1 -->
            <e1     name = "elem1"   id="elem1"    />
          </doc>
          <!-- Comment 2 -->
          <?pi-without-data ?>
        <!-- Comment 3 -->
      eoxml
    end

    it "retuns a canonicalized xml with '#{method}' method" do
      c = Xmldsig::Canonicalizer.new(doc, method).canonicalize

      expect(c).to match(/<e1 id=\"elem1\" name=\"elem1\"><\/e1>/)
      expect(c).to match(/Hello, world/)
      expect(c).to_not match(/Comment/)
      expect(c).to_not match(/DOCTYPE/)
    end
  end

  describe "#canonicalize" do
    context "when method is 'REC-xml-c14n-20010315'" do
      it_should_behave_like "a canonicalized xml", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315"
    end

    context "when method is 'xml-exc-c14n#'" do
      it_should_behave_like "a canonicalized xml", "http://www.w3.org/2001/10/xml-exc-c14n#"
    end

    context "when method is 'xml-c14n11'" do
      it_should_behave_like "a canonicalized xml", "http://www.w3.org/2006/12/xml-c14n11"
    end
  end
end
