require "spec_helper"

describe Xmldsig::Reference do
  let(:document) { Nokogiri::XML::Document.parse File.read("spec/xmldsig/fixtures/signed.xml") }
  let(:reference) { Xmldsig::Reference.new(document.at_xpath("//ds:Reference", Xmldsig::NAMESPACES)) }

  describe "#digest_value" do
    it "returns the digest value in the xml" do
      expect(reference.digest_value).to eql Base64.decode64("ftoSYFdze1AWgGHF5N9i9SFKThXkqH2AdyzA3/epbJw=")
    end
  end

  describe "#document" do
    it "returns the document" do
      expect(reference.document).to eql document
    end
  end

  describe "#sign" do
    let(:document) { Nokogiri::XML::Document.parse File.read("spec/xmldsig/fixtures/unsigned.xml") }

    it "sets the correct digest value" do
      reference.sign
      expect(reference.digest_value).to eql Base64.decode64("ftoSYFdze1AWgGHF5N9i9SFKThXkqH2AdyzA3/epbJw=")
    end
  end

  describe "#referenced_node" do
    it "returns the referenced_node by id" do
      expect(reference.referenced_node.to_s).to eql document.
        at_xpath("//*[@ID='foo']").to_s
    end

    it "returns the referenced node by parent" do
      allow(reference).to receive(:reference_uri).and_return("")
      expect(reference.referenced_node.to_s).to eql document.root.to_s
    end

    it "returns the reference node when using WS-Security style id attribute" do
      node = document.at_xpath("//*[@ID]")
      node.add_namespace("wsu", Xmldsig::NAMESPACES[:wsu])
      node["wsu:Id"] = node["ID"]
      node.remove_attribute("ID")
      expect(reference.referenced_node.attribute_with_ns("Id",
        Xmldsig::NAMESPACES[:wsu]).value).to eql "foo"
    end

    it "raises ReferencedNodeNotFound when the refenced node is not present" do
      node = document.at_xpath("//*[@ID]")
      node.remove_attribute("ID")

      expect { reference.referenced_node }.
        to raise_error(Xmldsig::Reference::ReferencedNodeNotFound)
    end
  end

  describe "#reference_uri" do
    it "returns the reference uri" do
      expect(reference.reference_uri).to eql "#foo"
    end
  end
end
