require "spec_helper"

describe Nfe::Models::Information::ProcRef do
  subject { build(:procRef) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :nProc }
  it { is_expected.to ensure_length_of(:nProc).is_at_least(1).is_at_most(60) }

  it { is_expected.to validate_presence_of :indProc }
  it { is_expected.to enumerize(:indProc).in(:sefaz, :justica_federal,
    :justica_estadual, :secex_rfb, :others) }

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<nProc>AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA</nProc>"
      xml += "<indProc>1</indProc>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Information::ProcRef.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.nProc).to eql "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
        expect(subject.indProc).to eql "justica_federal"
      end

      it { is_expected.to be_valid }
    end
  end
end
